import { Color, ContentView, ImageSource, Property } from '@nativescript/core';
export * from './geo.utils';
export * from './expression/expression-parser';
export * from './layers/layer-factory';
export * from './layers/parser/property-parser';
export declare const MapboxTraceCategory = "NativescriptMapbox";
export declare enum CLogTypes {
    log = 0,
    info = 1,
    warning = 2,
    error = 3
}
export declare const CLog: (type: CLogTypes, ...args: any[]) => void;
export declare enum MapStyle {
    DARK = "dark",
    OUTDOORS = "outdoors",
    LIGHT = "light",
    SATELLITE = "satellite",
    SATELLITE_STREETS = "satellite_streets",
    STREETS = "streets",
    TRAFFIC_DAY = "traffic_day",
    TRAFFIC_NIGHT = "traffic_night"
}
export interface LatLng {
    lat: number;
    lng: number;
}
export interface QueryRenderedFeaturesOptions {
    point?: LatLng;
    layers?: string[];
    filter?: any[];
}
export interface QuerySourceFeaturesOptions {
    sourceLayer?: string;
    filter?: any[];
}
export interface Feature {
    id?: any;
    type: string;
    geometry: Object;
    properties: Object;
}
export interface AddPolygonOptions {
    /**
     * Set this in case you want to later pass it to 'removePolygons'. TODO doesn't exist yet ;)
     */
    id?: any;
    points: LatLng[];
    fillColor?: string | Color;
    /**
     * Transparency / alpha, ranging from 0 to 1.
     * Default fully opaque (1).
     */
    fillOpacity?: number;
    /**
     * The line around the polygon. Barely visible on Android.
     */
    strokeColor?: string | Color;
    /**
     * iOS only.
     */
    strokeWidth?: number;
    /**
     * iOS only.
     */
    strokeOpacity?: number;
}
export interface UserLocation {
    location: LatLng;
    speed: number;
}
export interface SetCenterOptions extends LatLng {
    animated?: boolean;
}
export interface AddPolylineOptions {
    /**
     * Set this in case you want to later pass it to 'removePolylines'.
     */
    id?: any;
    /**
     * Width of the line, default 5.
     */
    width?: number;
    /**
     * Color of the line, default black.
     */
    color?: string | Color;
    /**
     * Transparency / alpha, ranging from 0 to 1.
     * Default fully opaque (1).
     */
    opacity?: number;
    points: LatLng[];
}
export interface MapboxMarker extends LatLng {
    /**
     * Set this in case you want to later pass it to 'removeMarker'.
     */
    id?: any;
    title?: string;
    subtitle?: string;
    /**
     * Prefix with 'res://' and load a file from the resources folder.
     * Details on how 'res://' is used can be found here: https://docs.nativescript.org/ui/images#load-images-from-resource
     * Example: "res://icon.file"
     */
    icon?: string;
    /**
     * The preferred way is using the 'icon' property, but you can still reference a local file directly.
     * Example: "~/assets/markers/green_pin_marker.png"
     */
    iconPath?: string;
    /**
     * A callback function to invoke when the marker is tapped.
     */
    onTap?: Function;
    /**
     * A callback function to invoke when the callout (popup) of this marker is tapped.
     */
    onCalloutTap?: Function;
    /**
     * Set to true to select the marker when rendered - effectively showing any configured callout.
     * Note that only 1 callout will be shown at any time on a Mapbox map.
     * Default false.
     */
    selected?: boolean;
    update?: (newSettings: MapboxMarker) => void;
    ios?: any;
    android?: any;
}
export interface SetZoomLevelOptions {
    level: number;
    animated: boolean;
}
export interface SetTiltOptions {
    /**
     * default 30 (degrees)
     */
    tilt: number;
    /**
     * default 5000 (milliseconds)
     */
    duration: number;
}
export interface ShowOptionsMargins {
    left?: number;
    right?: number;
    top?: number;
    bottom?: number;
}
export interface Bounds {
    north: number;
    east: number;
    south: number;
    west: number;
}
export interface Viewport {
    bounds: Bounds;
    zoomLevel: number;
}
export interface SetViewportOptions {
    bounds: Bounds;
    /**
     * Add an animation of about 1 second.
     * Default true.
     */
    animated?: boolean;
    /**
     * Optional padding.
     */
    padding?: number | {
        top?: number;
        left?: number;
        right?: number;
        bottom?: number;
    };
}
export interface DeleteOfflineRegionOptions {
    /**
     * The id of the offline region to delete.
     */
    id?: string;
    /**
     * The name of the offline region to delete.
     */
    name?: string;
}
export interface MapboxCluster {
    points: number;
    color: string;
}
export interface AddGeoJsonClusteredOptions {
    /**
     * A unique identifier, like: "earthquakes"
     */
    name: string;
    /**
     * URL, like: "https://www.mapbox.com/mapbox-gl-js/assets/earthquakes.geojson"
     */
    data: string;
    clusterMaxZoom?: number;
    clusterRadius?: number;
    clusters?: MapboxCluster[];
}
export type LayerType = 'fill' | 'line' | 'symbol' | 'circle' | 'heatmap' | 'fill-extrusion' | 'raster' | 'hillshade' | 'background' | 'sky';
export type SupportedLayerType = LayerType & ('line' | 'circle' | 'fill' | 'symbol' | 'raster');
export interface AddLayerOptions {
    id: string;
    source: string;
    sourceLayer: string;
    type: SupportedLayerType;
    /**
     * 'circle' paint properties
     */
    circleColor?: string | Color;
    circleOpacity?: number;
    circleRadius?: number;
    circleStrokeColor?: string | Color;
    circleStrokeWidth?: number;
    /**
     * 'fill' paint properties
     */
    fillColor?: string | Color;
    fillOpacity?: number;
    /**
     * 'line' layout properties
     */
    lineCap?: string;
    lineJoin?: string;
    /**
     * 'line' paint properties
     */
    lineColor?: string | Color;
    lineOpacity?: number;
    lineWidth?: number;
}
export type UserTrackingMode = 'NONE' | 'FOLLOW' | 'FOLLOW_WITH_HEADING' | 'FOLLOW_WITH_COURSE';
export type AddSourceOptions = VectorSource | GeoJSONSource | RasterSource;
export type UpdateSourceOptions = VectorSource | GeoJSONSource | RasterSource;
export interface Source {
    type: 'vector' | 'raster' | 'geojson';
    scheme?: 'xyz' | 'tms';
}
export interface RasterSource extends Source {
    type: 'raster';
    tiles: string[];
    bounds?: number[];
    minzoom?: number;
    maxzoom?: number;
    tileSize?: number;
}
export interface VectorSource extends Source {
    type: 'vector';
    url?: string;
    tiles?: string[];
    bounds?: number[];
    minzoom?: number;
    maxzoom?: number;
    tileSize?: number;
}
export interface GeoJSONSource extends Source {
    type: 'geojson';
    data?: any;
    minzoom?: number;
    maxzoom?: number;
    lineMetrics?: boolean;
    cluster?: {
        radius: any;
        maxZoom: any;
        properties?: {
            [property: string]: [any, any];
        };
    };
}
export type UserLocationCameraMode = 'NONE' | 'NONE_COMPASS' | 'NONE_GPS' | 'TRACKING' | 'TRACKING_COMPASS' | 'TRACKING_GPS' | 'TRACKING_GPS_NORTH';
export interface TrackUserOptions {
    cameraMode: UserLocationCameraMode;
    renderMode?: string;
    /**
     * iOS only, as Android is always animated. Default true (because of Android).
     */
    animated?: boolean;
}
export interface AddExtrusionOptions {
}
export interface OfflineRegion {
    name: string;
    bounds: Bounds;
    minZoom: number;
    maxZoom: number;
    style: MapStyle;
    metadata?: any;
    pixelRatio?: any;
    type?: any;
}
export interface DownloadProgress {
    name: string;
    completed: number;
    expected: number;
    percentage: number;
    complete: boolean;
    /**
     * Android only, the size in bytes of the download so far.
     */
    completedSize?: number;
}
export interface DownloadOfflineRegionOptions extends OfflineRegion {
    onProgress?: (data: DownloadProgress) => void;
    /**
     * Optional, used on Android only.
     * Set this, in case no map has been show yet (and thus, no accessToken has been passed in yet).
     */
    accessToken?: string;
}
export interface ListOfflineRegionsOptions {
    /**
     * Optional, used on Android only.
     * Set this, in case no map has been show yet (and thus, no accessToken has been passed in yet).
     */
    accessToken?: string;
}
export declare enum ControlPosition {
    TOP_LEFT = 0,
    TOP_RIGHT = 1,
    BOTTOM_LEFT = 2,
    BOTTOM_RIGHT = 3
}
/**
 * The options object passed into the show function.
 */
export interface ShowOptions {
    accessToken: string;
    /**
     * default 'streets'
     */
    style?: string | MapStyle;
    margins?: ShowOptionsMargins;
    center?: Partial<LatLng>;
    /**
     * default 0 (which is almost the entire planet)
     */
    zoomLevel?: number;
    /**
     * default false (true requires adding `NSLocationWhenInUseUsageDescription` or `NSLocationAlwaysUsageDescription` to the .plist)
     */
    showUserLocation?: boolean;
    /**
     * default false (required for the 'starter' plan)
     */
    hideLogo?: boolean;
    /**
     * default BOTTOM_LEFT
     */
    logoPosition?: ControlPosition;
    /**
     * default true
     */
    hideAttribution?: boolean;
    /**
     * default BOTTOM_LEFT
     */
    attributionPosition?: ControlPosition;
    /**
     * default false
     */
    hideCompass?: boolean;
    /**
     * default TOP_RIGHT
     */
    compassPosition?: ControlPosition;
    /**
     * default false
     */
    disableRotation?: boolean;
    /**
     * default false
     */
    disableScroll?: boolean;
    /**
     * default false
     */
    disableZoom?: boolean;
    /**
     * default false
     */
    disableTilt?: boolean;
    /**
     * Immediately add markers to the map
     */
    markers?: MapboxMarker[];
    /**
     * callback on location permission granted
     *
     * Android Only
     */
    onLocationPermissionGranted?: any;
    /**
     * callback on location permission denied
     *
     * Android Only
     */
    onLocationPermissionDenied?: any;
    /**
     * callback on Map Ready
     */
    onMapReady?: any;
    /**
     * callback on scroll event
     */
    onScrollEvent?: any;
    /**
     * callback on move begin event
     */
    onMoveBeginEvent?: any;
    /**
     * Android context
     */
    context?: any;
    /**
     * Android parent View
     */
    parentView?: any;
    /**
     * On Android by default there is a 200ms delay before showing the map to work around a race condition.
     */
    delay?: number;
    /**
     * See https://docs.mapbox.com/archive/android/maps/api/9.0.0/com/mapbox/mapboxsdk/location/LocationComponentOptions.html
     */
    locationComponentOptions: any;
}
export interface ShowResult {
    ios: any;
    android: any;
}
export interface AnimateCameraOptions {
    target?: LatLng;
    bounds?: Bounds;
    padding?: number;
    zoomLevel?: number;
    /**
     * For iOS, in meters from the ground
     */
    altitude?: number;
    bearing?: number;
    tilt?: number;
    duration?: number;
}
export interface LayerCommon {
    id: string;
    visibility(): boolean;
    show(): void;
    hide(): void;
    getNativeInstance(): any;
    setFilter(filter: any[]): void;
    getFilter(): any[];
    type(): LayerType;
}
export interface MapboxCommonApi {
    requestFineLocationPermission(): Promise<any>;
    hasFineLocationPermission(): Promise<boolean>;
}
export interface MapboxApi {
    setMapboxViewInstance(mapboxNativeViewInstance: any): void;
    initEventHandlerShim(settings: any, mapboxNativeViewInstance: any): void;
    onMapEvent(eventName: any, id: any, callback: any, nativeMapView?: any): void;
    offMapEvent(eventName: any, id: any, nativeMapView?: any): void;
    show(options: ShowOptions): Promise<ShowResult>;
    hide(): Promise<any>;
    unhide(): Promise<any>;
    destroy(nativeMap?: any): Promise<any>;
    onStart(nativeMap?: any): Promise<any>;
    onResume(nativeMap?: any): Promise<any>;
    onPause(nativeMap?: any): Promise<any>;
    onStop(nativeMap?: any): Promise<any>;
    onLowMemory(nativeMap?: any): Promise<any>;
    onDestroy(nativeMap?: any): Promise<any>;
    setMapStyle(style: string | MapStyle, nativeMap?: any): Promise<any>;
    addMarkers(markers: MapboxMarker[], nativeMap?: any): Promise<any>;
    removeMarkers(options?: any, nativeMap?: any): Promise<any>;
    setCenter(options: SetCenterOptions, nativeMap?: any): Promise<any>;
    getCenter(nativeMap?: any): Promise<LatLng>;
    setZoomLevel(options: SetZoomLevelOptions, nativeMap?: any): Promise<any>;
    getZoomLevel(nativeMap?: any): Promise<number>;
    setTilt(options: SetTiltOptions, nativeMap?: any): Promise<any>;
    getTilt(nativeMap?: any): Promise<number>;
    getUserLocation(nativeMap?: any): Promise<UserLocation>;
    showUserLocationMarker(options: any, nativeMap?: any): void;
    hideUserLocationMarker(nativeMap?: any): void;
    changeUserLocationMarkerMode(renderModeString: any, cameraModeString: UserLocationCameraMode, nativeMap?: any): void;
    forceUserLocationUpdate(location: any, nativeMap?: any): void;
    trackUser(options: TrackUserOptions, nativeMap?: any): Promise<void>;
    getUserLocationCameraMode(nativeMap?: any): UserLocationCameraMode;
    addSource(id: string, options: AddSourceOptions, nativeMapView?: any): Promise<any>;
    updateSource(id: string, options: UpdateSourceOptions, nativeMapView?: any): Promise<any>;
    removeSource(id: string, nativeMap?: any): Promise<any>;
    removeSource(id: string, nativeMap?: any): Promise<any>;
    addLayer(style: any, belowLayerId?: string, nativeMapView?: any): Promise<any>;
    addLayer(options: AddLayerOptions, belowLayerId?: string): Promise<any>;
    removeLayer(id: string, nativeMapView?: any): Promise<any>;
    removeLayer(id: string, nativeMap?: any): Promise<any>;
    addLinePoint(id: string, point: any, sourceId?: any, nativeMapView?: any): Promise<any>;
    queryRenderedFeatures(options: QueryRenderedFeaturesOptions, nativeMap?: any): Promise<Feature[]>;
    querySourceFeatures(sourceId: string, options?: QuerySourceFeaturesOptions, nativeMap?: any): Promise<Feature[]>;
    addPolygon(options: AddPolygonOptions, nativeMap?: any): Promise<any>;
    removePolygons(ids?: any[], nativeMap?: any): Promise<any>;
    addPolyline(options: AddPolylineOptions, nativeMap?: any): Promise<any>;
    removePolylines(ids?: any[], nativeMap?: any): Promise<any>;
    animateCamera(options: AnimateCameraOptions, nativeMap?: any): Promise<any>;
    setOnMapClickListener(listener: (data: LatLng) => boolean, nativeMap?: any): Promise<any>;
    setOnMapLongClickListener(listener: (data: LatLng) => boolean, nativeMap?: any): Promise<any>;
    setOnScrollListener(listener: (data?: LatLng) => void, nativeMap?: any): Promise<void>;
    setOnMoveBeginListener(listener: (data?: LatLng) => void, nativeMap?: any): Promise<void>;
    setOnMoveEndListener(listener: (data?: LatLng) => void, nativeMap?: any): Promise<void>;
    setOnFlingListener(listener: () => void, nativeMap?: any): Promise<any>;
    setOnCameraMoveListener(listener: (reason: any, animated?: boolean) => void, nativeMap?: any): Promise<any>;
    setOnCameraMoveCancelListener(listener: () => void, nativeMap?: any): Promise<any>;
    setOnCameraIdleListener(listener: () => void, nativeMap?: any): Promise<any>;
    requestFineLocationPermission(): Promise<any>;
    hasFineLocationPermission(): Promise<boolean>;
    getViewport(nativeMap?: any): Promise<Viewport>;
    setViewport(options: SetViewportOptions, nativeMap?: any): Promise<any>;
    downloadOfflineRegion(options: DownloadOfflineRegionOptions): Promise<any>;
    listOfflineRegions(options?: ListOfflineRegionsOptions): Promise<OfflineRegion[]>;
    deleteOfflineRegion(options: DeleteOfflineRegionOptions): Promise<any>;
    addGeoJsonClustered(options: AddGeoJsonClusteredOptions): Promise<any>;
    getLayer(name: string, nativeMap?: any): Promise<LayerCommon>;
    getLayers(nativeMap?: any): Promise<LayerCommon[]>;
    getImage(imageId: string, nativeMap?: any): Promise<ImageSource>;
    addImage(imageId: string, imagePath: string, nativeMap?: any): Promise<void>;
    removeImage(imageId: string, nativeMap?: any): Promise<void>;
    project(data: LatLng): {
        x: number;
        y: number;
    };
    projectBack(point: {
        x: number;
        y: number;
    }): LatLng;
}
export declare abstract class MapboxCommon implements MapboxCommonApi {
    view?: MapboxViewCommonBase;
    constructor(view?: MapboxViewCommonBase);
    static defaults: Partial<ShowOptions>;
    static merge(obj1: {}, obj2: {}): any;
    requestFineLocationPermission(): Promise<any>;
    hasFineLocationPermission(): Promise<boolean>;
    protected fetchImageSource(imagePath: string): Promise<ImageSource>;
}
/**
 * Interface definition for a View of a mapbox map.
 */
export interface MapboxViewApi {
    addMarkers(markers: MapboxMarker[]): Promise<any>;
    onMapEvent(eventName: any, id: any, callback: any): void;
    offMapEvent(eventName: any, id: any): void;
    removeMarkers(options?: any): Promise<any>;
    queryRenderedFeatures(options: QueryRenderedFeaturesOptions): Promise<Feature[]>;
    querySourceFeatures(sourceId: string, options?: QuerySourceFeaturesOptions): Promise<Feature[]>;
    setOnMapClickListener(listener: (data: LatLng) => boolean): Promise<any>;
    setOnMapLongClickListener(listener: (data: LatLng) => boolean): Promise<any>;
    setOnScrollListener(listener: (data?: LatLng) => void): Promise<void>;
    setOnMoveBeginListener(listener: (data?: LatLng) => void): Promise<void>;
    setOnFlingListener(listener: () => void): Promise<any>;
    setOnCameraMoveListener(listener: (reason: any, animated?: boolean) => void): Promise<any>;
    setOnCameraMoveCancelListener(listener: () => void): Promise<any>;
    setOnCameraIdleListener(listener: () => void): Promise<any>;
    getViewport(): Promise<Viewport>;
    setViewport(options: SetViewportOptions): Promise<any>;
    setMapStyle(style: string | MapStyle): Promise<any>;
    getCenter(): Promise<LatLng>;
    setCenter(options: SetCenterOptions): Promise<any>;
    getZoomLevel(): Promise<number>;
    setZoomLevel(options: SetZoomLevelOptions): Promise<any>;
    getTilt(): Promise<number>;
    setTilt(options: SetTiltOptions): Promise<any>;
    getUserLocation(): Promise<UserLocation>;
    trackUser(options: TrackUserOptions): Promise<any>;
    getUserLocationCameraMode(nativeMap?: any): UserLocationCameraMode;
    showUserLocationMarker(options: any): void;
    hideUserLocationMarker(options: any): void;
    changeUserLocationMarkerMode(renderModeString: any, cameraModeString: UserLocationCameraMode): void;
    forceUserLocationUpdate(location: any): void;
    addSource(id: string, options: AddSourceOptions): Promise<any>;
    updateSource(id: string, options: UpdateSourceOptions): Promise<any>;
    removeSource(id: string, nativeMap?: any): Promise<any>;
    addLayer(style: any, belowLayerId?: string): Promise<any>;
    removeLayer(id: string): Promise<any>;
    addLinePoint(id: string, point: any, sourceId?: any): Promise<any>;
    addPolygon(options: AddPolygonOptions): Promise<any>;
    removePolygons(ids?: any[]): Promise<any>;
    addPolyline(options: AddPolylineOptions): Promise<any>;
    removePolylines(ids?: any[]): Promise<any>;
    animateCamera(options: AnimateCameraOptions): Promise<any>;
    getLayer(name: string, nativeMap?: any): Promise<LayerCommon>;
    getLayers(nativeMap?: any): Promise<LayerCommon[]>;
    getImage(imageId: string, nativeMap?: any): Promise<ImageSource>;
    addImage(imageId: string, imagePath: string, nativeMap?: any): Promise<void>;
    removeImage(imageId: string, nativeMap?: any): Promise<void>;
    destroy(): Promise<any>;
    onStart(): Promise<any>;
    onResume(): Promise<any>;
    onPause(): Promise<any>;
    onStop(): Promise<any>;
    onLowMemory(): Promise<any>;
    onDestroy(): Promise<any>;
    project(data: LatLng): {
        x: number;
        y: number;
    };
    projectBack(screenCoordinate: {
        x: number;
        y: number;
    }): LatLng;
}
/**
 * common base for views created in XML.
 *
 * Instead of returning a reference to the Mapbox API instance (class Mapbox) from the view
 * the author decided to implement shim methods for a subset of the API. I'm not sure what the
 * reasoning was.
 *
 * @see Mapbox
 */
export declare abstract class MapboxViewCommonBase extends ContentView implements MapboxViewApi {
    protected mapbox: MapboxApi;
    telemetry: boolean;
    abstract getNativeMapView(): any;
    /**
     * map event
     *
     * The base NativeScript ContentView class has on() and off() methods.
     */
    onMapEvent(eventName: any, id: any, callback: any): void;
    offMapEvent(eventName: any, id: any): void;
    addMarkers(markers: MapboxMarker[]): Promise<any>;
    removeMarkers(options?: any): Promise<any>;
    setOnMapClickListener(listener: (data: LatLng) => boolean): Promise<any>;
    setOnMapLongClickListener(listener: (data: LatLng) => boolean): Promise<any>;
    setOnScrollListener(listener: (data?: LatLng) => void, nativeMap?: any): Promise<void>;
    setOnMoveBeginListener(listener: (data?: LatLng) => void, nativeMap?: any): Promise<void>;
    setOnMoveEndListener(listener: (data?: LatLng) => void, nativeMap?: any): Promise<void>;
    setOnFlingListener(listener: () => void, nativeMap?: any): Promise<any>;
    setOnCameraMoveListener(listener: (reason: any, animated?: boolean) => void, nativeMap?: any): Promise<any>;
    setOnCameraMoveCancelListener(listener: () => void, nativeMap?: any): Promise<any>;
    setOnCameraIdleListener(listener: () => void, nativeMap?: any): Promise<any>;
    getViewport(): Promise<Viewport>;
    setViewport(options: SetViewportOptions): Promise<any>;
    setMapStyle(style: string | MapStyle): Promise<any>;
    getCenter(): Promise<LatLng>;
    setCenter(options: SetCenterOptions): Promise<any>;
    getZoomLevel(): Promise<number>;
    setZoomLevel(options: SetZoomLevelOptions): Promise<any>;
    getTilt(): Promise<number>;
    setTilt(options: SetTiltOptions): Promise<any>;
    getUserLocation(): Promise<UserLocation>;
    showUserLocationMarker(options: any): void;
    hideUserLocationMarker(): void;
    changeUserLocationMarkerMode(renderModeString: any, cameraModeString: UserLocationCameraMode): void;
    forceUserLocationUpdate(location: any): void;
    trackUser(options: TrackUserOptions): Promise<any>;
    getUserLocationCameraMode(): UserLocationCameraMode;
    addSource(id: string, options: AddSourceOptions): Promise<any>;
    updateSource(id: string, options: UpdateSourceOptions): Promise<any>;
    removeSource(id: string): Promise<any>;
    addLayer(style: any, belowLayerId?: string): Promise<any>;
    removeLayer(id: string): Promise<any>;
    addLinePoint(id: string, point: any, sourceId?: any): Promise<any>;
    queryRenderedFeatures(options: QueryRenderedFeaturesOptions): Promise<Feature[]>;
    querySourceFeatures(sourceId: string, options?: QuerySourceFeaturesOptions): Promise<Feature[]>;
    addPolygon(options: AddPolygonOptions): Promise<any>;
    removePolygons(ids?: any[]): Promise<any>;
    addPolyline(options: AddPolylineOptions): Promise<any>;
    removePolylines(ids?: any[]): Promise<any>;
    animateCamera(options: AnimateCameraOptions): Promise<any>;
    getLayer(name: string, nativeMap?: any): Promise<LayerCommon>;
    getLayers(nativeMap?: any): Promise<LayerCommon[]>;
    getImage(imageId: string): Promise<ImageSource>;
    addImage(imageId: string, imagePath: string): Promise<void>;
    removeImage(imageId: string): Promise<void>;
    destroy(): Promise<any>;
    onStart(): Promise<any>;
    onResume(nativeMap?: any): Promise<any>;
    onPause(nativeMap?: any): Promise<any>;
    onStop(nativeMap?: any): Promise<any>;
    onLowMemory(nativeMap?: any): Promise<any>;
    onDestroy(nativeMap?: any): Promise<any>;
    project(data: LatLng): {
        x: number;
        y: number;
    };
    projectBack(screenCoordinate: {
        x: number;
        y: number;
    }): LatLng;
}
/**
 * Properties definitions for the Mapbox XML tag.
 *
 * @link https://docs.nativescript.org/plugins/ui-plugin-custom
 */
export declare const mapReadyProperty: Property<MapboxViewCommonBase, string>;
export declare const zoomLevelProperty: Property<MapboxViewCommonBase, number>;
export declare const accessTokenProperty: Property<MapboxViewCommonBase, string>;
export declare const mapStyleProperty: Property<MapboxViewCommonBase, string>;
export declare const latitudeProperty: Property<MapboxViewCommonBase, number>;
export declare const longitudeProperty: Property<MapboxViewCommonBase, number>;
export declare const showUserLocationProperty: Property<MapboxViewCommonBase, boolean>;
export declare const locationComponentOptionsProperty: Property<MapboxViewCommonBase, object>;
export declare const hideLogoProperty: Property<MapboxViewCommonBase, boolean>;
export declare const logoPositionProperty: Property<MapboxViewCommonBase, ControlPosition>;
export declare const hideAttributionProperty: Property<MapboxViewCommonBase, boolean>;
export declare const attributionPositionProperty: Property<MapboxViewCommonBase, ControlPosition>;
export declare const telemetryProperty: Property<MapboxViewCommonBase, boolean>;
export declare const hideCompassProperty: Property<MapboxViewCommonBase, boolean>;
export declare const compassPositionProperty: Property<MapboxViewCommonBase, ControlPosition>;
export declare const disableZoomProperty: Property<MapboxViewCommonBase, boolean>;
export declare const disableRotationProperty: Property<MapboxViewCommonBase, boolean>;
export declare const disableScrollProperty: Property<MapboxViewCommonBase, boolean>;
export declare const disableTiltProperty: Property<MapboxViewCommonBase, boolean>;
export declare const delayProperty: Property<MapboxViewCommonBase, number>;
/**
 * base class for views created in XML
 *
 * This is the glue that creates a config object based on the XML attributes passed to
 * the Mapbox XML tag.
 *
 * @see MapboxView
 *
 * @link https://docs.nativescript.org/plugins/ui-plugin-custom
 */
export declare abstract class MapboxViewBase extends MapboxViewCommonBase {
    static mapReadyEvent: string;
    static scrollEvent: string;
    static moveBeginEvent: string;
    static moveEndEvent: string;
    static locationPermissionGrantedEvent: string;
    static locationPermissionDeniedEvent: string;
    protected config: Partial<ShowOptions>;
}
