import { Color, File, Http, ImageSource, Trace, Utils, knownFolders, path } from '@nativescript/core';
import { CLog, CLogTypes, ControlPosition, MapStyle, MapboxCommon, MapboxViewBase, telemetryProperty } from './common';
import { Layer, LayerFactory } from './layers/layer-factory';
import { ExpressionParser } from './expression/expression-parser';
var MGLMapViewDelegateImpl = /** @class */ (function (_super) {
    __extends(MGLMapViewDelegateImpl, _super);
    function MGLMapViewDelegateImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MGLMapViewDelegateImpl.new = function () {
        return _super.new.call(this);
    };
    /**
     * initialize with the mapReady callback
     */
    MGLMapViewDelegateImpl.prototype.initWithCallback = function (mapLoadedCallback) {
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'MGLMapViewDelegateImpl::initWithCallback()');
        }
        this.mapLoadedCallback = mapLoadedCallback;
        return this;
    };
    /**
     * set a reference to the mapboxAPI instance
     */
    MGLMapViewDelegateImpl.prototype.setMapboxApi = function (api) {
        this.mapboxApi = api;
    };
    /**
     * set the user location click listener callback
     */
    MGLMapViewDelegateImpl.prototype.setUserLocationClickListener = function (callback) {
        this.userLocationClickListener = callback;
    };
    /**
     * set the user location click listener callback
     */
    MGLMapViewDelegateImpl.prototype.setUserLocationChangedistener = function (callback) {
        this.userLocationChangedListener = callback;
    };
    /**
     * set user location marker modes
     */
    MGLMapViewDelegateImpl.prototype.changeUserLocationRenderMode = function (userLocationRenderMode) {
        // nothing to do here
    };
    /**
     * set the camera changd listener callback
     */
    MGLMapViewDelegateImpl.prototype.setCameraChangedListener = function (callback) {
        this.cameraChangedListener = callback;
    };
    /**
     * set the camera idled listener callback
     */
    MGLMapViewDelegateImpl.prototype.setCameraIdledListener = function (callback) {
        this.cameraIdledListener = callback;
    };
    /**
     * set style loaded callback.
     *
     * set an optional callback to be invoked when a style set with
     * setMapStyle() is finished loading
     *
     * @param {function} callback function with loaded style as parameter.
     *
     * @see Mapbox:setMapStyle()
     */
    MGLMapViewDelegateImpl.prototype.setStyleLoadedCallback = function (callback) {
        this.styleLoadedCallback = callback;
    };
    /**
     * map ready callback
     */
    MGLMapViewDelegateImpl.prototype.mapViewDidFinishLoadingMap = function (mapView) {
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'MGLMapViewDelegateImpl:mapViewDidFinishLoadingMap(): top');
        }
        if (this.mapLoadedCallback !== undefined) {
            this.mapLoadedCallback(mapView);
            // this should be fired only once, but it's also fired when the style changes, so just remove the callback
            this.mapLoadedCallback = undefined;
        }
    };
    MGLMapViewDelegateImpl.prototype.mapViewDidFinishRenderingMapFullyRendered = function (mapView, fullyRendered) {
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'MGLMapViewDelegateImpl:mapViewDidFinishRenderingMapFullyRendered(): rendered is:', fullyRendered);
        }
    };
    /**
     * Callback when the style has been loaded.
     *
     * Based on my testing, it looks like this callback is invoked multiple times.
     *
     * @see Mapbox:setMapStyle()
     *
     * @link https://mapbox.github.io/mapbox-gl-native/macos/0.3.0/Protocols/MGLMapViewDelegate.html#/c:objc(pl)MGLMapViewDelegate(im)mapView:didFinishLoadingStyle:
     */
    MGLMapViewDelegateImpl.prototype.mapViewDidFinishLoadingStyle = function (mapView, style) {
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'MGLMapViewDelegateImpl:mapViewDidFinishLoadingStyle(): callback called.');
        }
        if (this.styleLoadedCallback !== undefined) {
            this.styleLoadedCallback(mapView, style);
            // to avoid multiple calls. This is only invoked from setMapStyle().
            this.styleLoadedCallback = undefined;
        }
    };
    /**
     * disable the default user location callout
     *
     * This took forever to find. The default iOS click handler for the user location
     * marker is about useless. It just displays "You Are Here". The examples do not
     * show how to disable it.
     */
    MGLMapViewDelegateImpl.prototype.mapViewAnnotationCanShowCallout = function (mapView, annotation) {
        if (annotation.isKindOfClass(MGLUserLocation.class())) {
            return false;
        }
        else {
            return true;
        }
    };
    MGLMapViewDelegateImpl.prototype.mapViewDidFailLoadingMapWithError = function (mapView, error) {
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'mapViewDidFailLoadingMapWithError: ' + error);
        }
    };
    MGLMapViewDelegateImpl.prototype.mapViewDidChangeUserTrackingModeAnimated = function (mapView, mode, animated) {
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'mapViewDidChangeUserTrackingModeAnimated: ' + mode);
        }
    };
    /**
     * fired when the marker icon is about to be rendered - return null for the default icon
     */
    MGLMapViewDelegateImpl.prototype.mapViewImageForAnnotation = function (mapView, annotation) {
        var cachedMarker = this.getTappedMarkerDetails(annotation);
        if (cachedMarker) {
            if (cachedMarker.reuseIdentifier) {
                var reusedImage = mapView.dequeueReusableAnnotationImageWithIdentifier(cachedMarker.reuseIdentifier);
                if (reusedImage) {
                    return reusedImage;
                }
            }
            // TODO try adding .rotatesToMatchCamera = true;
            // .. for instance in the mapViewDidDeselectAnnotationView / mapViewDidSelectAnnotationView / mapViewViewForAnnotation delegate
            if (cachedMarker.icon) {
                if (cachedMarker.icon.startsWith('res://')) {
                    var resourceName = cachedMarker.icon.substring('res://'.length);
                    var imageSource = ImageSource.fromResourceSync(resourceName);
                    if (imageSource === null) {
                        console.log("Unable to locate ".concat(resourceName));
                    }
                    else {
                        cachedMarker.reuseIdentifier = cachedMarker.icon;
                        return MGLAnnotationImage.annotationImageWithImageReuseIdentifier(imageSource.ios, cachedMarker.reuseIdentifier);
                    }
                }
                else if (cachedMarker.icon.startsWith('http')) {
                    if (cachedMarker.iconDownloaded !== null) {
                        cachedMarker.reuseIdentifier = cachedMarker.icon;
                        return MGLAnnotationImage.annotationImageWithImageReuseIdentifier(cachedMarker.iconDownloaded, cachedMarker.reuseIdentifier);
                    }
                }
                else {
                    if (Trace.isEnabled()) {
                        CLog(CLogTypes.info, 'Please use res://resourceName, http(s)://imageUrl or iconPath to use a local path');
                    }
                }
            }
            else if (cachedMarker.iconPath) {
                var appPath = knownFolders.currentApp().path;
                var iconFullPath = appPath + '/' + cachedMarker.iconPath.replace('~/', '');
                if (File.exists(iconFullPath)) {
                    var image = ImageSource.fromFileSync(iconFullPath).ios;
                    // perhaps add resize options for nice retina rendering (although you can now use the 'icon' param instead)
                    cachedMarker.reuseIdentifier = cachedMarker.iconPath;
                    return MGLAnnotationImage.annotationImageWithImageReuseIdentifier(image, cachedMarker.reuseIdentifier);
                }
            }
        }
        return null;
    };
    /**
     * fired when one of the callout's accessoryviews is tapped (not currently used)
     */
    MGLMapViewDelegateImpl.prototype.mapViewAnnotationCalloutAccessoryControlTapped = function (mapView, annotation, control) { };
    /**
     * fired when a marker is tapped
     */
    MGLMapViewDelegateImpl.prototype.mapViewDidSelectAnnotation = function (mapView, annotation) {
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'MGLMapViewDelegateImpl::mapViewDidSelectAnntation()');
        }
        if (annotation.isKindOfClass(MGLUserLocation.class())) {
            if (Trace.isEnabled()) {
                CLog(CLogTypes.info, 'MGLMapViewDelegateImpl::mapViewDidSelectAnnotation(): tapped the user location button');
            }
            if (typeof this.userLocationClickListener != 'undefined') {
                this.userLocationClickListener(annotation);
                return;
            }
            mapView.deselectAnnotationAnimated(annotation, false);
        }
        var cachedMarker = this.getTappedMarkerDetails(annotation);
        if (cachedMarker && cachedMarker.onTap) {
            cachedMarker.onTap(cachedMarker);
        }
    };
    /**
     * fired when a callout is tapped
     */
    MGLMapViewDelegateImpl.prototype.mapViewTapOnCalloutForAnnotation = function (mapView, annotation) {
        var cachedMarker = this.getTappedMarkerDetails(annotation);
        if (cachedMarker && cachedMarker.onCalloutTap) {
            cachedMarker.onCalloutTap(cachedMarker);
        }
    };
    MGLMapViewDelegateImpl.prototype.getTappedMarkerDetails = function (tapped) {
        for (var m in _markers) {
            var cached = _markers[m];
            // don't compare lat/lng types as they're not the same (same for (sub)title, they may be null vs undefined)
            if (
            // eslint-disable-next-line eqeqeq
            cached.lat == tapped.coordinate.latitude &&
                // eslint-disable-next-line eqeqeq
                cached.lng == tapped.coordinate.longitude &&
                // eslint-disable-next-line eqeqeq
                cached.title == tapped.title &&
                // eslint-disable-next-line eqeqeq
                cached.subtitle == tapped.subtitle) {
                return cached;
            }
        }
    };
    MGLMapViewDelegateImpl.prototype.mapViewRegionIsChangingWithReason = function (mapView, reason) {
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'MGLMapViewDelegateImpl::mapViewRegionIsChanging()');
        }
        if (this.cameraChangedListener) {
            this.cameraChangedListener(reason);
        }
    };
    MGLMapViewDelegateImpl.prototype.mapViewRegionDidChangeWithReasonAnimated = function (mapView, reason, animated) {
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'MGLMapViewDelegateImpl::mapViewRegionDidChangeAnimated()');
        }
        if (this.cameraChangedListener) {
            this.cameraChangedListener(reason, animated);
        }
        if (this.cameraIdledListener) {
            this.cameraIdledListener();
        }
    };
    MGLMapViewDelegateImpl.prototype.mapViewDidUpdateUserLocation = function (mapView, userLocation) {
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'MGLMapViewDelegateImpl::mapViewDidUpdateUserLocation()');
        }
        if (this.userLocationChangedListener) {
            this.userLocationChangedListener(_getLocation(userLocation));
        }
    };
    MGLMapViewDelegateImpl.ObjCProtocols = [MGLMapViewDelegate];
    return MGLMapViewDelegateImpl;
}(NSObject));
var MapTapHandlerImpl = /** @class */ (function (_super) {
    __extends(MapTapHandlerImpl, _super);
    function MapTapHandlerImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MapTapHandlerImpl.initWithOwnerAndListenerForMap = function (owner, listener, mapView) {
        var handler = MapTapHandlerImpl.new();
        handler._owner = owner;
        handler._listener = listener;
        handler._mapView = mapView;
        return handler;
    };
    MapTapHandlerImpl.prototype.tap = function (recognizer) {
        var tapPoint = recognizer.locationInView(this._mapView);
        var tapCoordinate = this._mapView.convertPointToCoordinateFromView(tapPoint, this._mapView);
        this._listener({
            lat: tapCoordinate.latitude,
            lng: tapCoordinate.longitude
        });
    };
    MapTapHandlerImpl.ObjCExposedMethods = {
        tap: { returns: interop.types.void, params: [interop.types.id] }
    };
    return MapTapHandlerImpl;
}(NSObject));
var MapLongPressHandlerImpl = /** @class */ (function (_super) {
    __extends(MapLongPressHandlerImpl, _super);
    function MapLongPressHandlerImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MapLongPressHandlerImpl.initWithOwnerAndListenerForMap = function (owner, listener, mapView) {
        var handler = MapLongPressHandlerImpl.new();
        handler._owner = owner;
        handler._listener = listener;
        handler._mapView = mapView;
        return handler;
    };
    MapLongPressHandlerImpl.prototype.longPress = function (recognizer) {
        var longPressPoint = recognizer.locationInView(this._mapView);
        var longPressCoordinate = this._mapView.convertPointToCoordinateFromView(longPressPoint, this._mapView);
        this._listener({
            lat: longPressCoordinate.latitude,
            lng: longPressCoordinate.longitude
        });
    };
    MapLongPressHandlerImpl.ObjCExposedMethods = {
        longPress: { returns: interop.types.void, params: [interop.types.id] }
    };
    return MapLongPressHandlerImpl;
}(NSObject));
var MapPanHandlerImpl = /** @class */ (function (_super) {
    __extends(MapPanHandlerImpl, _super);
    function MapPanHandlerImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MapPanHandlerImpl.initWithOwnerAndListenerForMap = function (owner, listener, panState, mapView) {
        var handler = MapPanHandlerImpl.new();
        handler._owner = owner;
        handler._listener = new Map([[panState, listener]]);
        handler._mapView = mapView;
        return handler;
    };
    MapPanHandlerImpl.prototype.addListener = function (panState, listener) {
        this._listener.set(panState, listener);
    };
    MapPanHandlerImpl.prototype.pan = function (recognizer) {
        var panCoordinate = this.getCoordinates(recognizer);
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'MapPanHandlerImpl::pan(): top with state:', recognizer.state);
        }
        if (recognizer.state === UIGestureRecognizerState.Changed) {
            this.notifyListener(recognizer.state, panCoordinate.latitude, panCoordinate.longitude);
        }
    };
    MapPanHandlerImpl.prototype.panEnd = function (recognizer) {
        var panCoordinate = this.getCoordinates(recognizer);
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'MapPanHandlerImpl::panEnd(): top with state:', recognizer.state);
        }
        if (recognizer.state === UIGestureRecognizerState.Ended) {
            this.notifyListener(recognizer.state, panCoordinate.latitude, panCoordinate.longitude);
        }
    };
    MapPanHandlerImpl.prototype.panBegin = function (recognizer) {
        var panCoordinate = this.getCoordinates(recognizer);
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'MapPanHandlerImpl::panBegin(): top with state:', recognizer.state);
        }
        if (recognizer.state === UIGestureRecognizerState.Began) {
            this.notifyListener(recognizer.state, panCoordinate.latitude, panCoordinate.longitude);
        }
    };
    MapPanHandlerImpl.prototype.getCoordinates = function (recognizer) {
        var panPoint = recognizer.locationInView(this._mapView);
        return this._mapView.convertPointToCoordinateFromView(panPoint, this._mapView);
    };
    MapPanHandlerImpl.prototype.notifyListener = function (panState, latitude, longitude) {
        if (this._listener.has(panState)) {
            this._listener.get(panState)({ lat: latitude, lng: longitude });
        }
    };
    MapPanHandlerImpl.ObjCExposedMethods = {
        pan: { returns: interop.types.void, params: [interop.types.id] },
        panEnd: { returns: interop.types.void, params: [interop.types.id] },
        panBegin: { returns: interop.types.void, params: [interop.types.id] }
    };
    return MapPanHandlerImpl;
}(NSObject));
var MapSwipeHandlerImpl = /** @class */ (function (_super) {
    __extends(MapSwipeHandlerImpl, _super);
    function MapSwipeHandlerImpl() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    MapSwipeHandlerImpl.initWithOwnerAndListenerForMap = function (owner, listener, mapView) {
        var handler = MapSwipeHandlerImpl.new();
        handler._owner = owner;
        handler._listener = listener;
        handler._mapView = mapView;
        return handler;
    };
    MapSwipeHandlerImpl.prototype.swipe = function (recognizer) {
        var swipePoint = recognizer.locationInView(this._mapView);
        var swipeCoordinate = this._mapView.convertPointToCoordinateFromView(swipePoint, this._mapView);
        this._listener({
            lat: swipeCoordinate.latitude,
            lng: swipeCoordinate.longitude
        });
    };
    MapSwipeHandlerImpl.ObjCExposedMethods = {
        swipe: { returns: interop.types.void, params: [interop.types.id] }
    };
    return MapSwipeHandlerImpl;
}(NSObject));
// Export the enums for devs not using TS
export * from './common';
let _markers = [];
const _markerIconDownloadCache = [];
const _setMapboxMapOptions = (mapView, settings) => {
    mapView.logoView.hidden = settings.hideLogo;
    mapView.logoViewPosition = _mapControlPositionToOrnamentPosition(settings.logoPosition);
    mapView.attributionButton.hidden = settings.hideAttribution;
    mapView.attributionButtonPosition = _mapControlPositionToOrnamentPosition(settings.attributionPosition);
    mapView.compassView.hidden = settings.hideCompass;
    mapView.compassViewPosition = _mapControlPositionToOrnamentPosition(settings.compassPosition);
    mapView.rotateEnabled = !settings.disableRotation;
    mapView.scrollEnabled = !settings.disableScroll;
    mapView.zoomEnabled = !settings.disableZoom;
    mapView.allowsTilting = !settings.disableTilt;
    if (settings.center && settings.center.lat && settings.center.lng) {
        const centerCoordinate = CLLocationCoordinate2DMake(settings.center.lat, settings.center.lng);
        mapView.setCenterCoordinateZoomLevelAnimated(centerCoordinate, settings.zoomLevel, false);
    }
    else {
        mapView.setZoomLevelAnimated(settings.zoomLevel, false);
    }
    mapView.showsUserLocation = settings.showUserLocation;
    mapView.autoresizingMask = 2 /* UIViewAutoresizing.FlexibleWidth */ | 16 /* UIViewAutoresizing.FlexibleHeight */;
};
const _mapControlPositionToOrnamentPosition = (position) => {
    switch (position) {
        case ControlPosition.TOP_LEFT:
            return 0 /* MGLOrnamentPosition.TopLeft */;
        case ControlPosition.TOP_RIGHT:
            return 1 /* MGLOrnamentPosition.TopRight */;
        case ControlPosition.BOTTOM_LEFT:
            return 2 /* MGLOrnamentPosition.BottomLeft */;
        case ControlPosition.BOTTOM_RIGHT:
            return 3 /* MGLOrnamentPosition.BottomRight */;
    }
};
const _getMapStyle = (input) => {
    if (input.startsWith('mapbox://styles') || input.startsWith('http://') || input.startsWith('https://')) {
        return NSURL.URLWithString(input);
    }
    else if (input.startsWith('~/')) {
        return NSURL.URLWithString('file://' + path.join(knownFolders.currentApp().path, input.replace('~/', '')));
    }
    else if (input === MapStyle.LIGHT) {
        return MGLStyle.lightStyleURL;
    }
    else if (input === MapStyle.DARK) {
        return MGLStyle.darkStyleURL;
    }
    else if (input === MapStyle.OUTDOORS) {
        return MGLStyle.outdoorsStyleURL;
    }
    else if (input === MapStyle.SATELLITE) {
        return MGLStyle.satelliteStyleURL;
    }
    else if (input === MapStyle.SATELLITE_STREETS) {
        return MGLStyle.satelliteStreetsStyleURL;
    }
    else if (input === MapStyle.TRAFFIC_DAY) {
        return NSURL.URLWithString('mapbox://styles/mapbox/traffic-day-v2');
    }
    else if (input === MapStyle.TRAFFIC_NIGHT) {
        return NSURL.URLWithString('mapbox://styles/mapbox/traffic-night-v2');
    }
    else {
        return MGLStyle.streetsStyleURL;
    }
};
function _getLocation(loc) {
    if (loc === null) {
        return null;
    }
    else {
        return {
            location: {
                lat: loc.coordinate.latitude,
                lng: loc.coordinate.longitude
            },
            speed: loc.location ? loc.location.speed : 0
        };
    }
}
export function setLogLevel(level) {
    let loggingLevel;
    switch (level) {
        case 'none':
            loggingLevel = 0 /* MGLLoggingLevel.None */;
            break;
        case 'info':
            loggingLevel = 1 /* MGLLoggingLevel.Info */;
            break;
        case 'verbose':
        case 'debug':
            loggingLevel = 2 /* MGLLoggingLevel.Debug */;
            break;
        case 'error':
            loggingLevel = 3 /* MGLLoggingLevel.Error */;
            break;
        case 'fault':
            loggingLevel = 4 /* MGLLoggingLevel.Fault */;
            break;
    }
    MGLLoggingConfiguration.sharedConfiguration.loggingLevel = loggingLevel;
}
/**
 * Map View Class instantiated from XML
 *
 * This class is created by the NativeScript XML view parsing
 * code.
 */
export class MapboxView extends MapboxViewBase {
    constructor() {
        super(...arguments);
        this.nativeMapView = null;
        this.delegate = null;
        this.settings = null;
        this.initialized = false;
        // see initMap. Count of how many times we've
        // tried to init the map.
        this.initCountHack = 50;
    }
    /**
     * programmatically include settings
     */
    setConfig(settings) {
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'setConfig(): settings:', settings);
        }
        this.settings = settings;
    }
    getNativeMapView() {
        return this.nativeMapView;
    }
    createNativeView() {
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'createNativeView(): top');
        }
        const v = super.createNativeView();
        return v;
    }
    /**
     * init the native view.
     *
     * FIXME: It appears that the order of events is different between iOS and Android.
     * In the demo under Android, the main-page event handler is called first then the one
     * in the plugin. Under iOS it's the reverse.
     *
     * The symptom is that any properties that reference a binding aren't available
     * at the time this method is called. For example {{access_token}}.
     *
     * I'm sure there is something I do not understand about how this is supposed to work
     * and that the handstands below are not necessary.
     */
    onLoaded() {
        super.onLoaded();
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'initNativeView(): on - loaded');
        }
        if (this.telemetry === false) {
            NSUserDefaults.standardUserDefaults.setBoolForKey(false, 'MGLMapboxMetricsEnabled');
        }
        if (!this.initialized) {
            this.initMap();
            this.initialized = true;
        }
    }
    initNativeView() {
        super.initNativeView();
        this.nativeView.owner = this;
    }
    /**
     * when the view is destroyed.
     *
     * This is called by the framework when the view is destroyed (made not visible).
     *
     * However, it does not seem to be called when the page is unloaded.
     *
     * @link https://docs.nativescript.org/plugins/ui-plugin-custom
     */
    async disposeNativeView() {
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'disposeNativeView(): top');
        }
        this.nativeView.owner = null;
        await this.mapbox.destroy();
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'disposeNativeView(): after mapbox.destroy()');
        }
        super.disposeNativeView();
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'disposeNativeView(): bottom');
        }
    }
    /**
     * returns a reference to the class Mapbox API shim instance
     *
     * @see Mapbox
     */
    getMapboxApi() {
        return this.mapbox;
    }
    /**
     * initialize the map
     *
     * @see MGLMapViewDelegateImpl
     *
     * @todo FIXME: figure out why the accessToken property (which is using a binding in the demo XML) isn't set before we arrive here.
     */
    initMap() {
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'initMap() top with settings:', this.settings);
        }
        // FIXME: HACK: if we are arriving here because of an XML parse the property evaluations may not have
        // happened yet. This needs to be redone, but for the moment we'll assume the accessToken is done
        // via a property eval (since it really shouldn't be hard coded in XML).
        //
        // settings will only be set here if we are programmatically showing a map.
        if (!this.settings && !this.config.accessToken) {
            if (Trace.isEnabled()) {
                CLog(CLogTypes.info, 'initMap() no access token. Race condition on XML property evaluation?');
            }
            // If the user didn't specify an accessToken we don't want to loop forever
            if (this.initCountHack > 50) {
                return;
            }
            // FIXME: super ugly.
            setTimeout(() => {
                this.initMap();
            }, 50);
            this.initCountHack++;
            return;
        }
        if (!this.settings) {
            this.settings = Mapbox.merge(this.config, Mapbox.defaults);
        }
        else {
            this.settings = Mapbox.merge(this.settings, Mapbox.defaults);
        }
        if (!this.nativeMapView) {
            this.mapbox = new Mapbox(this);
            if (Trace.isEnabled()) {
                CLog(CLogTypes.info, 'initMap(): after new Mapbox()');
            }
            // called in a setTimeout call at the bottom.
            const drawMap = () => {
                MGLAccountManager.accessToken = this.settings.accessToken;
                this.nativeMapView = MGLMapView.alloc().initWithFrameStyleURL(CGRectMake(0, 0, this.nativeView.frame.size.width, this.nativeView.frame.size.height), _getMapStyle(this.settings.style));
                // this delegate class is defined later in this file and is where, in Obj-C land,
                // callbacks are delivered and handled.
                this.nativeMapView.delegate = this.delegate = MGLMapViewDelegateImpl.new().initWithCallback(() => {
                    if (Trace.isEnabled()) {
                        CLog(CLogTypes.info, 'initMap(): MLMapViewDeleteImpl onMapReady callback');
                    }
                    // FIXME: on the Android side the view is created in Mapbox::show(). On the iOS side it's created
                    // here in MapboxView, however the mapbox api still needs a reference to it.
                    this.mapbox.setMapboxViewInstance(this.nativeMapView);
                    this.mapbox.initEventHandlerShim(this.settings, this.nativeMapView);
                    this.notify({
                        eventName: MapboxViewBase.mapReadyEvent,
                        object: this,
                        map: this,
                        ios: this.nativeMapView
                    });
                    // no permission required, but to align with Android we fire the event anyway
                    this.notify({
                        eventName: MapboxViewBase.locationPermissionGrantedEvent,
                        object: this,
                        map: this,
                        ios: this.nativeMapView
                    });
                });
                // this.delegate.setStyleLoadedCallback((map, style)=>{
                //     this.delegate.setStyleLoadedCallback(null);
                // });
                _setMapboxMapOptions(this.nativeMapView, this.settings);
                _markers = [];
                this.nativeView.addSubview(this.nativeMapView);
                // this.notify will notify an event listener specified
                // in the XML, in this case (onMoveBegin)="..."
                this.mapbox.setOnMoveBeginListener((data) => {
                    if (Trace.isEnabled()) {
                        CLog(CLogTypes.info, 'initMap(): onMoveBegin listener');
                    }
                    this.notify({
                        eventName: MapboxViewBase.moveBeginEvent,
                        object: this,
                        map: this,
                        ios: this.nativeMapView
                    });
                }, this.nativeMapView);
                this.mapbox.setOnMoveEndListener((data) => {
                    if (Trace.isEnabled()) {
                        CLog(CLogTypes.info, 'initMap(): onMoveEnd listener');
                    }
                    this.notify({
                        eventName: MapboxViewBase.moveEndEvent,
                        object: this,
                        map: this,
                        ios: this.nativeMapView
                    });
                }, this.nativeMapView);
                this.mapbox.setOnScrollListener((data) => {
                    if (Trace.isEnabled()) {
                        CLog(CLogTypes.info, 'initMap(): onScroll listener');
                    }
                    this.notify({
                        eventName: MapboxViewBase.scrollEvent,
                        object: this,
                        map: this,
                        ios: this.nativeMapView
                    });
                }, this.nativeMapView);
            };
            // draw the map after a timeout
            setTimeout(drawMap, this.settings.delay ? this.settings.delay : 0);
        }
    } // end of initMap()
    onLayout(left, top, right, bottom) {
        super.onLayout(left, top, right, bottom);
        if (this.nativeMapView) {
            this.nativeMapView.layer.frame = this.ios.layer.bounds;
        }
    }
    [telemetryProperty.setNative](value) {
        NSUserDefaults.standardUserDefaults.setBoolForKey(false, 'MGLMapboxMetricsEnabled');
    }
}
export class Mapbox extends MapboxCommon {
    constructor() {
        // reference to the native mapbox API
        super(...arguments);
        this.eventCallbacks = {};
    }
    /**
     * set the mapboxViewInstance
     *
     * @see MapboxView::initMap();
     */
    setMapboxViewInstance(mapboxViewInstance) {
        this._mapboxViewInstance = mapboxViewInstance;
    }
    /**
     * event handler shim
     *
     * Initialize our event handler shim so that we can intercept events here.
     *
     * @param { MapboxView } mapboxView
     */
    initEventHandlerShim(settings, mapboxNativeViewInstance) {
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'Mapbox:initEventHandlerShim(): top');
        }
        this.setOnMapClickListener((point) => this.checkForClickEvent(point), mapboxNativeViewInstance);
    }
    /**
     * register a map event handler
     *
     * The NativeScript ContentView base class as on() and off() methods.
     */
    onMapEvent(eventName, id, callback, nativeMapView) {
        if (typeof this.eventCallbacks[eventName] == 'undefined') {
            this.eventCallbacks[eventName] = [];
        }
        this.eventCallbacks[eventName].push({
            id,
            callback
        });
    }
    offMapEvent(eventName, id, nativeMapView) {
        if (typeof this.eventCallbacks[eventName] == 'undefined') {
            return;
        }
        this.eventCallbacks[eventName] = this.eventCallbacks[eventName].filter((entry) => entry.id !== id);
    }
    /**
     * If click events registered and a feature found for the event, then fire listener.
     */
    checkForClickEvent(point, nativeMap) {
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'Mapbox:checkForClickEvent(): got click event with point:', point);
        }
        this.eventCallbacks['click'] &&
            this.eventCallbacks['click'].forEach((eventListener) => {
                this.queryRenderedFeatures({ point, layers: [eventListener.id] }, nativeMap)
                    .then((response) => {
                    if (response.length > 0) {
                        eventListener.callback(response);
                    }
                })
                    .catch((err) => {
                    console.error('click error ', eventListener.id, err);
                });
            });
        this.view && this.view.notify({ eventName: 'mapClick', object: this.view, point });
        return false;
    }
    _addMarkers(markers, nativeMap) {
        if (!markers) {
            if (Trace.isEnabled()) {
                CLog(CLogTypes.info, 'No markers passed');
            }
            return;
        }
        if (!Array.isArray(markers)) {
            if (Trace.isEnabled()) {
                CLog(CLogTypes.info, "markers must be passed as an Array: [{title: 'foo'}]");
            }
            return;
        }
        const theMap = nativeMap || this._mapboxViewInstance;
        _downloadMarkerImages(markers).then((updatedMarkers) => {
            updatedMarkers.forEach((marker) => {
                const lat = marker.lat;
                const lng = marker.lng;
                const point = MGLPointAnnotation.new();
                point.coordinate = CLLocationCoordinate2DMake(lat, lng);
                point.title = marker.title;
                point.subtitle = marker.subtitle;
                // needs to be done before adding to the map, otherwise the delegate method 'mapViewImageForAnnotation' can't use it
                _markers.push(marker);
                theMap.addAnnotation(point);
                if (marker.selected) {
                    theMap.selectAnnotationAnimated(point, false);
                }
                marker.ios = point;
                marker.update = (newSettings) => {
                    _markers.forEach((_marker) => {
                        if (marker.id === _marker.id) {
                            if (newSettings.onTap !== undefined) {
                                _marker.onTap = newSettings.onTap;
                            }
                            if (newSettings.onCalloutTap !== undefined) {
                                _marker.onCalloutTap = newSettings.onCalloutTap;
                            }
                            if (newSettings.title !== undefined) {
                                _marker.ios.title = _marker.title = newSettings.title;
                            }
                            if (newSettings.subtitle !== undefined) {
                                _marker.ios.subtitle = _marker.subtitle = newSettings.subtitle;
                            }
                            if (newSettings.lat && newSettings.lng) {
                                _marker.lat = newSettings.lat;
                                _marker.lng = newSettings.lng;
                                _marker.ios.coordinate = CLLocationCoordinate2DMake(newSettings.lat, newSettings.lng);
                            }
                            if (newSettings.selected) {
                                theMap.selectAnnotationAnimated(_marker.ios, false);
                            }
                        }
                    });
                };
            });
        });
    }
    /**
     * create an display the map
     *
     * @todo FIXME: This method is not called. See MapboxView::initMap().
     */
    show(options) {
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'show(): top with options:', options);
        }
        return new Promise((resolve, reject) => {
            try {
                const settings = Mapbox.merge(options, Mapbox.defaults);
                // let directions = MBDirections.alloc().initWithAccessToken(arg.accessToken);
                // alert("directions: " + directions);
                // if no accessToken was set the app may crash
                if (settings.accessToken === undefined) {
                    reject("Please set the 'accessToken' parameter");
                    return;
                }
                // if already added, make sure it's removed first
                if (this._mapboxViewInstance) {
                    this._mapboxViewInstance.removeFromSuperview();
                }
                const view = UIApplication.sharedApplication.keyWindow.rootViewController.view, frameRect = view.frame, mapFrame = CGRectMake(settings.margins.left, settings.margins.top, frameRect.size.width - settings.margins.left - settings.margins.right, frameRect.size.height - settings.margins.top - settings.margins.bottom), styleURL = _getMapStyle(settings.style);
                MGLAccountManager.accessToken = settings.accessToken;
                this._mapboxViewInstance = MGLMapView.alloc().initWithFrameStyleURL(mapFrame, styleURL);
                _setMapboxMapOptions(this._mapboxViewInstance, settings);
                this._mapboxViewInstance.delegate = MGLMapViewDelegateImpl.new().initWithCallback((mapView) => {
                    resolve({
                        ios: mapView
                    });
                });
                _markers = [];
                this._addMarkers(settings.markers);
                // wrapping in a little timeout since the map area tends to flash black a bit initially
                setTimeout(() => {
                    view.addSubview(this._mapboxViewInstance);
                }, 500);
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.show: ' + ex);
                }
                reject(ex);
            }
        });
    }
    hide() {
        return new Promise((resolve, reject) => {
            try {
                if (this._mapboxViewInstance) {
                    this._mapboxViewInstance.removeFromSuperview();
                }
                resolve();
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.hide: ' + ex);
                }
                reject(ex);
            }
        });
    }
    unhide() {
        return new Promise((resolve, reject) => {
            try {
                if (this._mapboxViewInstance) {
                    const view = UIApplication.sharedApplication.keyWindow.rootViewController.view;
                    view.addSubview(this._mapboxViewInstance);
                    resolve();
                }
                else {
                    reject('No map found');
                }
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.unhide: ' + ex);
                }
                reject(ex);
            }
        });
    }
    destroy(nativeMap) {
        return new Promise((resolve, reject) => {
            const theMap = nativeMap || this._mapboxViewInstance;
            if (theMap) {
                theMap.removeFromSuperview();
                theMap.delegate = null;
            }
            resolve();
        });
    }
    // ----------------------------------------
    // Life Cycle Hooks - Required on Android
    // ----------------------------------------
    onStart(nativeMap) {
        return Promise.resolve();
    }
    onResume(nativeMap) {
        return Promise.resolve();
    }
    onPause(nativeMap) {
        return Promise.resolve();
    }
    onStop(nativeMap) {
        return Promise.resolve();
    }
    onLowMemory(nativeMap) {
        return Promise.resolve();
    }
    onDestroy(nativeMap) {
        return Promise.resolve();
    }
    /**
     * explicitly set a map style
     */
    setMapStyle(style, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                // the style takes some time to load so we have to set a callback
                // to wait for the style to finish loading
                const delegate = theMap.delegate;
                delegate.setStyleLoadedCallback((mapView) => {
                    if (Trace.isEnabled()) {
                        CLog(CLogTypes.info, 'Mapbox:setMapStyle(): style loaded callback returned.');
                    }
                    resolve();
                });
                theMap.styleURL = _getMapStyle(style);
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.setMapStyle: ' + ex);
                }
                reject(ex);
            }
        });
    }
    async getImage(imageId, nativeMap) {
        return new Promise((resolve, reject) => {
            const theMap = nativeMap || this._mapboxViewInstance;
            if (!theMap) {
                reject('No map has been loaded');
                return;
            }
            try {
                const nativeImage = theMap.style.imageForName(imageId);
                const img = new ImageSource(nativeImage);
                resolve(img);
            }
            catch (ex) {
                reject('Error during getImage: ' + ex);
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.getImage: ' + ex);
                }
                throw ex;
            }
        });
    }
    async addImage(imageId, imagePath, nativeMap) {
        return new Promise(async (resolve, reject) => {
            const theMap = nativeMap || this._mapboxViewInstance;
            if (!theMap) {
                reject('No map has been loaded');
                return;
            }
            try {
                const imageSource = await this.fetchImageSource(imagePath);
                theMap.style.setImageForName(imageSource.ios, imageId);
                resolve();
            }
            catch (ex) {
                reject('Error during addImage: ' + ex);
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.addImage: ' + ex);
                }
                throw ex;
            }
        });
    }
    async removeImage(imageId, nativeMap) {
        return new Promise((resolve, reject) => {
            const theMap = nativeMap || this._mapboxViewInstance;
            if (!theMap) {
                reject('No map has been loaded');
                return;
            }
            try {
                theMap.style.removeImageForName(imageId);
                resolve();
            }
            catch (ex) {
                reject('Error during removeImage: ' + ex);
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.removeImage: ' + ex);
                }
                throw ex;
            }
        });
    }
    addMarkers(markers, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                this._addMarkers(markers, theMap);
                resolve();
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.addMarkers: ' + ex);
                }
                reject(ex);
            }
        });
    }
    removeMarkers(ids, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                const markersToRemove = [];
                _markers.forEach((marker) => {
                    if (!ids || (marker.id && ids.indexOf(marker.id) > -1)) {
                        markersToRemove.push(marker.ios);
                    }
                });
                // remove markers from cache
                if (ids) {
                    _markers = _markers.filter((marker) => ids.indexOf(marker.id) < 0);
                }
                else {
                    _markers = [];
                }
                if (markersToRemove.length > 0) {
                    theMap.removeAnnotations(markersToRemove);
                }
                resolve();
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.removeMarkers: ' + ex);
                }
                reject(ex);
            }
        });
    }
    setCenter(options, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                const animated = options.animated === undefined || options.animated;
                const coordinate = CLLocationCoordinate2DMake(options.lat, options.lng);
                theMap.setCenterCoordinateAnimated(coordinate, animated);
                resolve();
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.setCenter: ' + ex);
                }
                reject(ex);
            }
        });
    }
    getCenter(nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                const coordinate = theMap.centerCoordinate;
                resolve({
                    lat: coordinate.latitude,
                    lng: coordinate.longitude
                });
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.getCenter: ' + ex);
                }
                reject(ex);
            }
        });
    }
    setZoomLevel(options, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                const animated = options.animated === undefined || options.animated;
                const level = options.level;
                if (level >= 0 && level <= 20) {
                    theMap.setZoomLevelAnimated(level, animated);
                    resolve();
                }
                else {
                    reject('invalid ZoomLevel, use any double value from 0 to 20 (like 8.3)');
                }
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.setZoomLevel: ' + ex);
                }
                reject(ex);
            }
        });
    }
    getZoomLevel(nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                resolve(theMap.zoomLevel);
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.getZoomLevel: ' + ex);
                }
                reject(ex);
            }
        });
    }
    setTilt(options, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                const cam = theMap.camera;
                cam.pitch = options.tilt;
                const durationMs = options.duration ? options.duration : 5000;
                theMap.setCameraWithDurationAnimationTimingFunction(cam, durationMs / 1000, CAMediaTimingFunction.functionWithName(kCAMediaTimingFunctionEaseInEaseOut));
                setTimeout(() => {
                    resolve();
                }, durationMs);
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.setTilt: ' + ex);
                }
                reject(ex);
            }
        });
    }
    getTilt(nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                resolve(theMap.camera.pitch);
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.getTilt: ' + ex);
                }
                reject(ex);
            }
        });
    }
    getUserLocation(nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                const loc = theMap.userLocation;
                if (loc === null) {
                    reject('Location not available');
                }
                else {
                    resolve(_getLocation(loc));
                }
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.getUserLocation: ' + ex);
                }
                reject(ex);
            }
        });
    }
    /**
     * convert string to camera mode constant.
     *
     * Supported modes on iOS are different than on Android.
     *
     * @todo come up with a reasonable set of cross platform defaults.
     */
    _stringToCameraMode(mode) {
        switch (mode) {
            case 'NONE':
                return 0 /* MGLUserTrackingMode.None */;
            case 'NONE_COMPASS':
                console.log('MapboxView::_stringToCameraMode(): NONE_COMPASS unsupported on iOS');
                return 0 /* MGLUserTrackingMode.None */;
            case 'NONE_GPS':
                console.log('MapboxView::_stringToCameraMode(): NONE_GPS unsupported on iOS');
                return 0 /* MGLUserTrackingMode.None */;
            case 'TRACKING':
                return 1 /* MGLUserTrackingMode.Follow */;
            case 'TRACKING_COMPASS':
                return 2 /* MGLUserTrackingMode.FollowWithHeading */;
            case 'TRACKING_GPS':
                // a reasonable approximation.
                return 1 /* MGLUserTrackingMode.Follow */;
            case 'TRACKING_GPS_NORTH':
                return 3 /* MGLUserTrackingMode.FollowWithCourse */;
            default:
                console.log(`_stringToCameraMode: invalid cameraMode: ${mode}`);
        }
        return 0 /* MGLUserTrackingMode.None */;
    }
    _stringToRenderMode(mode) {
        let renderMode;
        switch (mode) {
            case 'NORMAL':
                return 'NORMAL';
            case 'COMPASS':
                return 'COMPASS';
            case 'GPS':
                return 'GPS';
        }
    }
    _convertCameraMode(mode) {
        switch (mode) {
            case 0 /* MGLUserTrackingMode.None */:
                return 'NONE';
            case 1 /* MGLUserTrackingMode.Follow */:
                return 'TRACKING';
            case 2 /* MGLUserTrackingMode.FollowWithHeading */:
                return 'TRACKING_COMPASS';
            case 3 /* MGLUserTrackingMode.FollowWithCourse */:
                return 'TRACKING_GPS_NORTH';
        }
    }
    /**
     * show a user location marker
     *
     * This method must not be called before location permissions have been granted.
     *
     * Supported options under iOS are:
     *
     * - renderMode
     * - cameraMode
     * - clickListener
     *
     * Other options are ignored. Compare with the android version that supports a
     * different set of options.
     *
     * @param {object} options
     */
    showUserLocationMarker(options, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                theMap.showsUserLocation = true;
                theMap.userTrackingMode = this._stringToCameraMode(options.cameraMode);
                theMap.showsUserHeadingIndicator = true;
                this.userLocationRenderMode = this._stringToRenderMode(options.renderMode);
                // the "delegate" needs to know the modes
                const delegate = theMap.delegate;
                // tell the delegate to tell the CustomerLocationAnnotationView to change the
                // appearance of the marker.
                delegate.changeUserLocationRenderMode(this.userLocationRenderMode);
                if (typeof options.clickListener != 'undefined') {
                    delegate.setUserLocationClickListener(options.clickListener);
                }
                resolve();
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.getUserLocation: ' + ex);
                }
                reject(ex);
            }
        });
    }
    /**
     * hide the user location marker
     *
     * @todo unfinished
     */
    hideUserLocationMarker(nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                resolve();
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.getUserLocation: ' + ex);
                }
                reject(ex);
            }
        });
    }
    /**
     * Change the mode of the user location marker
     *
     * Used to change the camera tracking and render modes of an existing
     * marker.
     *
     * The marker must be configured using showUserLocationMarker before this method
     * can called.
     */
    changeUserLocationMarkerMode(renderModeString, cameraModeString, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, "Mapbox::changeUserLocationMarkerMode(): changing renderMode to '" + renderModeString + "' cameraMode '" + cameraModeString + "'");
                }
                theMap.userTrackingMode = this._stringToCameraMode(cameraModeString);
                const delegate = theMap.delegate;
                const renderMode = this._stringToRenderMode(renderModeString);
                delegate.changeUserLocationRenderMode(renderMode);
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.showUserLocationMarker: ' + ex);
                }
                reject(ex);
            }
        });
    }
    /**
     * ignored on iOS
     */
    forceUserLocationUpdate(location, nativeMap) { }
    queryRenderedFeatures(options, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                if (options.point === undefined) {
                    reject("Please set the 'point' parameter");
                    return;
                }
                if (!options) {
                    options = {};
                }
                const { x, y } = theMap.convertCoordinateToPointToView({ latitude: options.point.lat, longitude: options.point.lng }, theMap);
                const queryLayerIds = options.layers ? NSSet.setWithArray(Utils.ios.collections.jsArrayToNSArray(options.layers)) : null;
                const queryFilter = options.filter ? ExpressionParser.parseJson(options.filter) : null;
                const features = theMap.visibleFeaturesAtPointInStyleLayersWithIdentifiersPredicate({ x, y }, queryLayerIds, queryFilter);
                const result = [];
                for (let i = 0; i < features.count; i++) {
                    const feature = features.objectAtIndex(i);
                    const featureJson = NSJSONSerialization.dataWithJSONObjectOptionsError(feature.geoJSONDictionary(), 0);
                    result.push(JSON.parse(NSString.alloc().initWithDataEncoding(featureJson, NSUTF8StringEncoding)));
                }
                resolve(result);
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.queryRenderedFeatures: ' + ex);
                }
                reject(ex);
            }
        });
    }
    querySourceFeatures(sourceId, options, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                if (!options) {
                    options = {};
                }
                const source = theMap.style.sourceWithIdentifier(sourceId);
                if (!source) {
                    throw new Error(`Source with id "${sourceId}" not found.`);
                }
                let features;
                const queryFilter = options.filter ? ExpressionParser.parseJson(options.filter) : null;
                if (source instanceof MGLShapeSource) {
                    features = source.featuresMatchingPredicate(queryFilter);
                }
                else if (source instanceof MGLVectorTileSource) {
                    if (!options.sourceLayer) {
                        throw new Error('The option "sourceLayer" is required for vector sources.');
                    }
                    const sourceLayerIds = options.sourceLayer ? NSSet.setWithArray(Utils.ios.collections.jsArrayToNSArray([options.sourceLayer])) : null;
                    features = source.featuresInSourceLayersWithIdentifiersPredicate(sourceLayerIds, queryFilter);
                }
                else {
                    throw new Error('Only sources from type "vector" or "geojson" are supported.');
                }
                const result = [];
                for (let i = 0; i < features.count; i++) {
                    const feature = features.objectAtIndex(i);
                    const featureJson = NSJSONSerialization.dataWithJSONObjectOptionsError(feature.geoJSONDictionary(), 0);
                    result.push(JSON.parse(NSString.alloc().initWithDataEncoding(featureJson, NSUTF8StringEncoding)));
                }
                resolve(result);
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.querySourceFeatures: ' + ex);
                }
                reject(ex);
            }
        });
    }
    /**
     * @deprecated
     */
    addPolygon(options, nativeMap) {
        return new Promise((resolve, reject) => {
            const theMap = nativeMap || this._mapboxViewInstance;
            const points = options.points;
            if (points === undefined) {
                reject("Please set the 'points' parameter");
                return;
            }
            const coordinateArray = [];
            points.forEach((point) => coordinateArray.push([point.lng, point.lat]));
            const polygonID = `polygon_${options.id || new Date().getTime()}`;
            if (theMap.style.sourceWithIdentifier(polygonID)) {
                reject("Remove the polygon with this id first with 'removePolygons': " + polygonID);
                return;
            }
            const geoJSON = `{
                "type": "FeatureCollection",
                "features": [
                {
                    "id": ${JSON.stringify(polygonID)},
                    "type": "Feature",
                    "properties": {
                    },
                    "geometry": {
                    "type": "Polygon",
                    "coordinates": [${JSON.stringify(coordinateArray)}]
                    }
                }
                ]
            }`;
            const geoDataStr = NSString.stringWithString(geoJSON);
            const geoData = geoDataStr.dataUsingEncoding(NSUTF8StringEncoding);
            const geoDataBase64Enc = geoData.base64EncodedStringWithOptions(0);
            const geo = NSData.alloc().initWithBase64EncodedStringOptions(geoDataBase64Enc, null);
            const shape = MGLShape.shapeWithDataEncodingError(geo, NSUTF8StringEncoding);
            const source = MGLShapeSource.alloc().initWithIdentifierShapeOptions(polygonID, shape, null);
            theMap.style.addSource(source);
            if (options.strokeColor || options.strokeWidth || options.strokeOpacity) {
                const strokeLayer = MGLLineStyleLayer.alloc().initWithIdentifierSource(polygonID + '_stroke', source);
                strokeLayer.lineColor = NSExpression.expressionForConstantValue(!options.strokeColor ? UIColor.blackColor : options.strokeColor instanceof Color ? options.strokeColor.ios : new Color(options.strokeColor).ios);
                strokeLayer.lineWidth = NSExpression.expressionForConstantValue(options.strokeWidth || 5);
                strokeLayer.lineOpacity = NSExpression.expressionForConstantValue(options.strokeOpacity === undefined ? 1 : options.strokeOpacity);
                theMap.style.addLayer(strokeLayer);
            }
            const layer = MGLFillStyleLayer.alloc().initWithIdentifierSource(polygonID, source);
            layer.fillColor = NSExpression.expressionForConstantValue(!options.fillColor ? UIColor.blackColor : options.fillColor instanceof Color ? options.fillColor.ios : new Color(options.fillColor).ios);
            layer.fillOpacity = NSExpression.expressionForConstantValue(options.fillOpacity === undefined ? 1 : options.fillOpacity);
            theMap.style.addLayer(layer);
            resolve();
        });
    }
    addPolyline(options, nativeMap) {
        return new Promise((resolve, reject) => {
            const theMap = nativeMap || this._mapboxViewInstance;
            const points = options.points;
            if (points === undefined) {
                reject("Please set the 'points' parameter");
                return;
            }
            const coordinateArray = [];
            points.forEach((point) => coordinateArray.push([point.lng, point.lat]));
            const polylineID = 'polyline_' + (options.id || new Date().getTime());
            // this would otherwise crash the app
            if (theMap.style.sourceWithIdentifier(polylineID)) {
                reject("Remove the polyline with this id first with 'removePolylines': " + polylineID);
                return;
            }
            const geoJSON = `{"type": "FeatureCollection", "features": [{"type": "Feature","properties": {},"geometry": {"type": "LineString", "coordinates": ${JSON.stringify(coordinateArray)}}}]}`;
            const geoDataStr = NSString.stringWithString(geoJSON);
            const geoData = geoDataStr.dataUsingEncoding(NSUTF8StringEncoding);
            const geoDataBase64Enc = geoData.base64EncodedStringWithOptions(0);
            const geo = NSData.alloc().initWithBase64EncodedStringOptions(geoDataBase64Enc, null);
            const shape = MGLShape.shapeWithDataEncodingError(geo, NSUTF8StringEncoding);
            const source = MGLShapeSource.alloc().initWithIdentifierShapeOptions(polylineID, shape, null);
            theMap.style.addSource(source);
            const layer = MGLLineStyleLayer.alloc().initWithIdentifierSource(polylineID, source);
            layer.lineColor = NSExpression.expressionForConstantValue(!options.color ? UIColor.blackColor : options.color instanceof Color ? options.color.ios : new Color(options.color).ios);
            layer.lineWidth = NSExpression.expressionForConstantValue(options.width || 5);
            layer.lineOpacity = NSExpression.expressionForConstantValue(options.opacity === undefined ? 1 : options.opacity);
            theMap.style.addLayer(layer);
            resolve();
        });
    }
    removePolyById(theMap, id) {
        let layer = theMap.style.layerWithIdentifier(id);
        if (layer !== null) {
            theMap.style.removeLayer(layer);
        }
        // polygons may have a 'stroke' layer
        layer = theMap.style.layerWithIdentifier(id + '_stroke');
        if (layer !== null) {
            theMap.style.removeLayer(layer);
        }
        const source = theMap.style.sourceWithIdentifier(id);
        if (source !== null) {
            theMap.style.removeSource(source);
        }
    }
    removePolys(polyType, ids, nativeMap) {
        return new Promise((resolve) => {
            const theMap = nativeMap || this._mapboxViewInstance;
            ids.forEach((id) => this.removePolyById(theMap, polyType + id));
            resolve();
        });
    }
    removePolygons(ids, nativeMap) {
        return this.removePolys('polygon_', ids, nativeMap);
    }
    removePolylines(ids, nativeMap) {
        return this.removePolys('polyline_', ids, nativeMap);
    }
    animateCamera(options, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                let cam;
                if (options.bounds) {
                    const padding = options.padding || 0;
                    // ensure padding is an object and assign default values
                    const { top = 0, left = 0, bottom = 0, right = 0 } = typeof padding === 'object' ? padding : { top: padding, left: padding, bottom: padding, right: padding };
                    // support defined padding
                    const insets = { top, left, bottom, right };
                    const bounds = {
                        sw: CLLocationCoordinate2DMake(options.bounds.south, options.bounds.west),
                        ne: CLLocationCoordinate2DMake(options.bounds.north, options.bounds.east)
                    };
                    cam = theMap.cameraThatFitsCoordinateBoundsEdgePadding(bounds, insets);
                }
                else {
                    const target = options.target;
                    if (target === undefined) {
                        reject("Please set the 'target' parameter");
                        return;
                    }
                    cam = theMap.camera;
                    cam.centerCoordinate = CLLocationCoordinate2DMake(target.lat, target.lng);
                }
                if (options.altitude) {
                    cam.altitude = options.altitude;
                }
                if (options.bearing) {
                    cam.heading = options.bearing;
                }
                if (options.tilt) {
                    cam.pitch = options.tilt;
                }
                if (options.zoomLevel && options.target) {
                    cam.altitude = MGLAltitudeForZoomLevel(options.zoomLevel, cam.pitch, options.target.lat, theMap.frame.size);
                }
                const durationMs = options.duration ? options.duration : 10000;
                theMap.setCameraWithDurationAnimationTimingFunction(cam, durationMs / 1000, CAMediaTimingFunction.functionWithName(kCAMediaTimingFunctionEaseInEaseOut));
                setTimeout(() => {
                    resolve();
                }, durationMs);
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.animateCamera: ' + ex);
                }
                reject(ex);
            }
        });
    }
    /**
     * sets a map level click listener
     *
     */
    setOnMapClickListener(listener, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                if (!theMap) {
                    reject('No map has been loaded');
                    return;
                }
                theMap['mapTapHandler'] = MapTapHandlerImpl.initWithOwnerAndListenerForMap(new WeakRef(this), listener, theMap);
                const tapGestureRecognizer = UITapGestureRecognizer.alloc().initWithTargetAction(theMap['mapTapHandler'], 'tap');
                for (let i = 0; i < theMap.gestureRecognizers.count; i++) {
                    const recognizer = theMap.gestureRecognizers.objectAtIndex(i);
                    if (recognizer instanceof UITapGestureRecognizer) {
                        tapGestureRecognizer.requireGestureRecognizerToFail(recognizer);
                    }
                }
                theMap.addGestureRecognizer(tapGestureRecognizer);
                resolve();
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.setOnMapClickListener: ' + ex);
                }
                reject(ex);
            }
        });
    }
    setOnMapLongClickListener(listener, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                if (!theMap) {
                    reject('No map has been loaded');
                    return;
                }
                // adding the longPress handler to the map oject so it's not GC'd
                theMap['mapLongPressHandler'] = MapLongPressHandlerImpl.initWithOwnerAndListenerForMap(new WeakRef(this), listener, theMap);
                const longPressGestureRecognizer = UILongPressGestureRecognizer.alloc().initWithTargetAction(theMap['mapLongPressHandler'], 'longPress');
                // cancel the default longPress handler
                for (let i = 0; i < theMap.gestureRecognizers.count; i++) {
                    const recognizer = theMap.gestureRecognizers.objectAtIndex(i);
                    if (recognizer instanceof UILongPressGestureRecognizer) {
                        longPressGestureRecognizer.requireGestureRecognizerToFail(recognizer);
                    }
                }
                theMap.addGestureRecognizer(longPressGestureRecognizer);
                resolve();
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.setOnMapClickListener: ' + ex);
                }
                reject(ex);
            }
        });
    }
    setOnScrollListener(listener, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                if (!theMap) {
                    reject('No map has been loaded');
                    return;
                }
                // adding the pan handler to the map oject so it's not GC'd
                if (theMap['mapPanHandler'] === undefined) {
                    theMap['mapPanHandler'] = MapPanHandlerImpl.initWithOwnerAndListenerForMap(new WeakRef(this), listener, 2 /* UIGestureRecognizerState.Changed */, theMap);
                }
                else {
                    theMap['mapPanHandler'].addListener(2 /* UIGestureRecognizerState.Changed */, listener);
                }
                // there's already a pan recognizer, so find it and attach a target action
                for (let i = 0; i < theMap.gestureRecognizers.count; i++) {
                    const recognizer = theMap.gestureRecognizers.objectAtIndex(i);
                    if (recognizer instanceof UIPanGestureRecognizer) {
                        recognizer.addTargetAction(theMap['mapPanHandler'], 'pan');
                        break;
                    }
                }
                resolve();
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.setOnScrollListener: ' + ex);
                }
                reject(ex);
            }
        });
    }
    /**
     * simulates onMoveBegin single event callback
     *
     * This will call the listener provided once per pan akin to the way
     * onMoveBegin on the Android side works.
     */
    setOnMoveBeginListener(listener, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                if (!theMap) {
                    reject('No map has been loaded');
                    return;
                }
                // adding the pan handler to the map oject so it's not GC'd
                if (theMap['mapPanHandler'] === undefined) {
                    theMap['mapPanHandler'] = MapPanHandlerImpl.initWithOwnerAndListenerForMap(new WeakRef(this), listener, 1 /* UIGestureRecognizerState.Began */, theMap);
                }
                else {
                    theMap['mapPanHandler'].addListener(1 /* UIGestureRecognizerState.Began */, listener);
                }
                // there's already a pan recognizer, so find it and attach a target action
                for (let i = 0; i < theMap.gestureRecognizers.count; i++) {
                    const recognizer = theMap.gestureRecognizers.objectAtIndex(i);
                    if (recognizer instanceof UIPanGestureRecognizer) {
                        recognizer.addTargetAction(theMap['mapPanHandler'], 'panBegin');
                        break;
                    }
                }
                resolve();
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.setOnMoveBeginListener: ' + ex);
                }
                reject(ex);
            }
        });
    }
    setOnMoveEndListener(listener, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                if (!theMap) {
                    reject('No map has been loaded');
                    return;
                }
                if (theMap['mapPanHandler'] === undefined) {
                    theMap['mapPanHandler'] = MapPanHandlerImpl.initWithOwnerAndListenerForMap(new WeakRef(this), listener, 3 /* UIGestureRecognizerState.Ended */, theMap);
                }
                else {
                    theMap['mapPanHandler'].addListener(3 /* UIGestureRecognizerState.Ended */, listener);
                }
                // there's already a pan recognizer, so find it and attach a target action
                for (let i = 0; i < theMap.gestureRecognizers.count; i++) {
                    const recognizer = theMap.gestureRecognizers.objectAtIndex(i);
                    if (recognizer instanceof UIPanGestureRecognizer) {
                        recognizer.addTargetAction(theMap['mapPanHandler'], 'panEnd');
                        break;
                    }
                }
                resolve();
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.setOnMoveEndListener: ' + ex);
                }
                reject(ex);
            }
        });
    }
    setOnFlingListener(listener, nativeMap) {
        // there's no swipe event we can bind to
        return Promise.reject("'setOnFlingListener' is not supported on iOS");
    }
    async setOnCameraMoveListener(listener, nativeMap) {
        const theMap = nativeMap || this._mapboxViewInstance;
        if (theMap) {
            theMap.delegate.setCameraChangedListener(listener);
        }
        else {
            return Promise.reject('No map has been loaded');
        }
    }
    setOnCameraMoveCancelListener(listener, nativeMap) {
        return Promise.reject("'setOnCameraMoveCancelListener' not currently supported on iOS");
    }
    async setOnCameraIdleListener(listener, nativeMap) {
        const theMap = nativeMap || this._mapboxViewInstance;
        if (theMap) {
            theMap.delegate.setCameraIdledListener(listener);
        }
        else {
            return Promise.reject('No map has been loaded');
        }
    }
    getViewport(nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                if (!theMap) {
                    reject('No map has been loaded');
                    return;
                }
                const visibleBounds = theMap.visibleCoordinateBounds;
                const bounds = {
                    north: visibleBounds.ne.latitude,
                    east: visibleBounds.ne.longitude,
                    south: visibleBounds.sw.latitude,
                    west: visibleBounds.sw.longitude
                };
                resolve({
                    bounds,
                    zoomLevel: theMap.zoomLevel
                });
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.getViewport: ' + ex);
                }
                reject(ex);
            }
        });
    }
    setViewport(options, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                if (!theMap) {
                    reject('No map has been loaded');
                    return;
                }
                const bounds = {
                    sw: CLLocationCoordinate2DMake(options.bounds.south, options.bounds.west),
                    ne: CLLocationCoordinate2DMake(options.bounds.north, options.bounds.east)
                };
                const animated = options.animated === undefined || options.animated;
                // define default padding
                const defaultPadding = 25;
                // check if padding is defined and whether it's an object or a single value
                const padding = options.padding !== undefined
                    ? typeof options.padding === 'object'
                        ? {
                            top: options.padding.top ?? 0,
                            left: options.padding.left ?? 0,
                            bottom: options.padding.bottom ?? 0,
                            right: options.padding.right ?? 0
                        }
                        : {
                            top: options.padding,
                            left: options.padding,
                            bottom: options.padding,
                            right: options.padding
                        }
                    : {
                        top: defaultPadding,
                        left: defaultPadding,
                        bottom: defaultPadding,
                        right: defaultPadding
                    };
                theMap.setVisibleCoordinateBoundsEdgePaddingAnimated(bounds, padding, animated);
                resolve();
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.setViewport: ' + ex);
                }
                reject(ex);
            }
        });
    }
    downloadOfflineRegion(options) {
        return new Promise((resolve, reject) => {
            try {
                const styleURL = _getMapStyle(options.style);
                const swCoordinate = CLLocationCoordinate2DMake(options.bounds.south, options.bounds.west);
                const neCoordinate = CLLocationCoordinate2DMake(options.bounds.north, options.bounds.east);
                const bounds = {
                    sw: swCoordinate,
                    ne: neCoordinate
                };
                const region = MGLTilePyramidOfflineRegion.alloc().initWithStyleURLBoundsFromZoomLevelToZoomLevel(styleURL, bounds, options.minZoom, options.maxZoom);
                if (options.accessToken) {
                    MGLAccountManager.accessToken = options.accessToken;
                }
                // TODO there's more observers, see https://www.mapbox.com/ios-sdk/examples/offline-pack/
                if (options.onProgress) {
                    _addObserver(MGLOfflinePackProgressChangedNotification, (notification) => {
                        const offlinePack = notification.object;
                        const offlinePackProgress = offlinePack.progress;
                        const userInfo = NSKeyedUnarchiver.unarchiveObjectWithData(offlinePack.context);
                        const complete = offlinePackProgress.countOfResourcesCompleted === offlinePackProgress.countOfResourcesExpected;
                        options.onProgress({
                            name: userInfo.objectForKey('name'),
                            completed: offlinePackProgress.countOfResourcesCompleted,
                            expected: offlinePackProgress.countOfResourcesExpected,
                            percentage: Math.round((offlinePackProgress.countOfResourcesCompleted / offlinePackProgress.countOfResourcesExpected) * 10000) / 100,
                            complete
                        });
                        if (complete) {
                            resolve();
                        }
                    });
                }
                _addObserver(MGLOfflinePackErrorNotification, (notification) => {
                    const offlinePack = notification.object;
                    const userInfo = NSKeyedUnarchiver.unarchiveObjectWithData(offlinePack.context);
                    const error = notification.userInfo[MGLOfflinePackUserInfoKeyError];
                    reject({
                        name: userInfo.objectForKey('name'),
                        error: 'Download error. ' + error
                    });
                });
                _addObserver(MGLOfflinePackMaximumMapboxTilesReachedNotification, (notification) => {
                    const offlinePack = notification.object;
                    const userInfo = NSKeyedUnarchiver.unarchiveObjectWithData(offlinePack.context);
                    const maximumCount = notification.userInfo[MGLOfflinePackUserInfoKeyMaximumCount];
                    console.log(`Offline region '${userInfo.objectForKey('name')}' reached the tile limit of ${maximumCount}`);
                });
                // Store some data for identification purposes alongside the downloaded resources.
                const userInfo = { name: options.name };
                const context = NSKeyedArchiver.archivedDataWithRootObject(userInfo);
                // Create and register an offline pack with the shared offline storage object.
                MGLOfflineStorage.sharedOfflineStorage.addPackForRegionWithContextCompletionHandler(region, context, (pack, error) => {
                    if (error) {
                        // The pack couldn’t be created for some reason.
                        reject(error.localizedFailureReason);
                    }
                    else {
                        // Start downloading.
                        pack.resume();
                    }
                });
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.downloadOfflineRegion: ' + ex);
                }
                reject(ex);
            }
        });
    }
    listOfflineRegions(options) {
        return new Promise((resolve, reject) => {
            try {
                const packs = MGLOfflineStorage.sharedOfflineStorage.packs;
                if (!packs) {
                    reject('No packs found or Mapbox not ready yet');
                    return;
                }
                const regions = [];
                for (let i = 0; i < packs.count; i++) {
                    const pack = packs.objectAtIndex(i);
                    const region = pack.region;
                    const userInfo = NSKeyedUnarchiver.unarchiveObjectWithData(pack.context);
                    regions.push({
                        name: userInfo.objectForKey('name'),
                        style: '' + region.styleURL,
                        minZoom: region.minimumZoomLevel,
                        maxZoom: region.maximumZoomLevel,
                        bounds: {
                            north: region.bounds.ne.latitude,
                            east: region.bounds.ne.longitude,
                            south: region.bounds.sw.latitude,
                            west: region.bounds.sw.longitude
                        }
                    });
                }
                resolve(regions);
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.listOfflineRegions: ' + ex);
                }
                reject(ex);
            }
        });
    }
    deleteOfflineRegion(options) {
        return new Promise((resolve, reject) => {
            try {
                if (!options || (!options.id && !options.name)) {
                    reject("Pass in the 'id' or 'name' param");
                    return;
                }
                const packs = MGLOfflineStorage.sharedOfflineStorage.packs;
                let found = false;
                for (let i = 0; i < packs.count; i++) {
                    const pack = packs.objectAtIndex(i);
                    const userInfo = NSKeyedUnarchiver.unarchiveObjectWithData(pack.context);
                    const regionId = options.id ? userInfo.objectForKey('id') : userInfo.objectForKey('name');
                    if (regionId === (options.id || options.name)) {
                        found = true;
                        MGLOfflineStorage.sharedOfflineStorage.removePackWithCompletionHandler(pack, (error) => {
                            if (error) {
                                // The pack couldn’t be deleted for some reason.
                                reject(error.localizedFailureReason);
                            }
                            else {
                                resolve();
                                // don't return, see note below
                            }
                        });
                        // don't break the loop as there may be multiple packs with the same name
                    }
                }
                if (!found) {
                    reject('Region not found');
                }
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.deleteOfflineRegion: ' + ex);
                }
                reject(ex);
            }
        });
    }
    addExtrusion(options, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                if (!theMap) {
                    reject('No map has been loaded');
                    return;
                }
                resolve();
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.deleteOfflineRegion: ' + ex);
                }
                reject(ex);
            }
        });
    }
    /**
     * update a geojson source
     *
     */
    updateSource(id, options, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                if (!theMap) {
                    reject('No map has been loaded');
                    return;
                }
                const source = theMap.style.sourceWithIdentifier(id);
                if (!source) {
                    reject('Source does not exists: ' + id);
                    return;
                }
                switch (options.type) {
                    case 'geojson':
                        const content = NSString.stringWithString(JSON.stringify(options.data));
                        const nsData = content.dataUsingEncoding(NSUTF8StringEncoding);
                        const geoJsonShape = MGLShape.shapeWithDataEncodingError(nsData, NSUTF8StringEncoding);
                        source.shape = geoJsonShape;
                        resolve();
                        break;
                    default:
                        reject('Invalid source type: ' + options['type']);
                        return;
                }
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.addSource: ' + ex);
                }
                reject(ex);
            }
        });
    }
    /**
     * add a vector or geojson source
     *
     * Add a source that can then be referenced in the style specification
     * passed to addLayer().
     *
     * @link https://docs.mapbox.com/mapbox-gl-js/api/#map#addsource
     */
    addSource(id, options, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                let source;
                if (!theMap) {
                    reject('No map has been loaded');
                    return;
                }
                if (theMap.style.sourceWithIdentifier(id)) {
                    reject('Source exists: ' + id);
                    return;
                }
                switch (options.type) {
                    case 'vector': {
                        if (options.url) {
                            source = MGLVectorTileSource.alloc().initWithIdentifierConfigurationURL(id, NSURL.URLWithString(options.url));
                        }
                        else {
                            const sourceOptions = {};
                            if (options.minzoom !== undefined) {
                                sourceOptions[MGLTileSourceOptionMinimumZoomLevel] = options.minzoom;
                            }
                            if (options.maxzoom !== undefined) {
                                sourceOptions[MGLTileSourceOptionMaximumZoomLevel] = options.maxzoom;
                            }
                            if (options.scheme) {
                                switch (options.scheme) {
                                    case 'xyz':
                                        sourceOptions[MGLTileSourceOptionTileCoordinateSystem] = 0 /* MGLTileCoordinateSystem.XYZ */;
                                        break;
                                    case 'tms':
                                        sourceOptions[MGLTileSourceOptionTileCoordinateSystem] = 1 /* MGLTileCoordinateSystem.TMS */;
                                        break;
                                    default:
                                        throw new Error('Unknown raster tile scheme.');
                                }
                            }
                            if (options.bounds) {
                                sourceOptions[MGLTileSourceOptionCoordinateBounds] = NSValue.valueWithMGLCoordinateBounds({
                                    sw: CLLocationCoordinate2DMake(options.bounds[1], options.bounds[0]),
                                    ne: CLLocationCoordinate2DMake(options.bounds[3], options.bounds[2])
                                });
                            }
                            source = MGLVectorTileSource.alloc().initWithIdentifierTileURLTemplatesOptions(id, options.tiles, sourceOptions);
                        }
                        break;
                    }
                    case 'geojson':
                        if (theMap.style.sourceWithIdentifier(id)) {
                            reject("Remove the layer with this id first with 'removeLayer': " + id);
                            return;
                        }
                        let geoJsonShape;
                        if (options.data) {
                            const content = NSString.stringWithString(JSON.stringify(options.data));
                            const nsData = content.dataUsingEncoding(NSUTF8StringEncoding);
                            geoJsonShape = MGLShape.shapeWithDataEncodingError(nsData, NSUTF8StringEncoding);
                        }
                        const sourceOptions = {};
                        if (options.minzoom !== undefined) {
                            sourceOptions[MGLShapeSourceOptionMinimumZoomLevel] = options.minzoom;
                        }
                        if (options.maxzoom !== undefined) {
                            sourceOptions[MGLShapeSourceOptionMaximumZoomLevel] = options.maxzoom;
                        }
                        if (options.lineMetrics !== undefined) {
                            sourceOptions[MGLShapeSourceOptionLineDistanceMetrics] = options.lineMetrics;
                        }
                        if (options.cluster) {
                            sourceOptions[MGLShapeSourceOptionClustered] = true;
                            sourceOptions[MGLShapeSourceOptionClusterRadius] = options.cluster.radius || 40;
                            sourceOptions[MGLShapeSourceOptionMaximumZoomLevelForClustering] = options.cluster.maxZoom || 13;
                            if (options.cluster.properties) {
                                const clusterProperties = {};
                                for (const property of Object.keys(options.cluster.properties)) {
                                    // eslint-disable-next-line prefer-const
                                    let [operator, operand] = options.cluster.properties[property];
                                    if (!Array.isArray(operator)) {
                                        operator = [operator];
                                    }
                                    const expressions = Utils.ios.collections.jsArrayToNSArray([ExpressionParser.parseJson(operator), ExpressionParser.parseJson(operand)]);
                                    clusterProperties[property] = expressions;
                                }
                                sourceOptions[MGLShapeSourceOptionClusterProperties] = clusterProperties;
                            }
                        }
                        source = MGLShapeSource.alloc().initWithIdentifierShapeOptions(id, geoJsonShape, sourceOptions);
                        break;
                    case 'raster': {
                        const sourceOptions = {
                            [MGLTileSourceOptionTileSize]: options.tileSize || 256
                        };
                        if (options.minzoom !== undefined) {
                            sourceOptions[MGLTileSourceOptionMinimumZoomLevel] = options.minzoom;
                        }
                        if (options.maxzoom !== undefined) {
                            sourceOptions[MGLTileSourceOptionMaximumZoomLevel] = options.maxzoom;
                        }
                        if (options.scheme) {
                            switch (options.scheme || 'xyz') {
                                case 'xyz':
                                    sourceOptions[MGLTileSourceOptionTileCoordinateSystem] = 0 /* MGLTileCoordinateSystem.XYZ */;
                                    break;
                                case 'tms':
                                    sourceOptions[MGLTileSourceOptionTileCoordinateSystem] = 1 /* MGLTileCoordinateSystem.TMS */;
                                    break;
                                default:
                                    throw new Error('Unknown raster tile scheme.');
                            }
                        }
                        if (options.bounds) {
                            sourceOptions[MGLTileSourceOptionCoordinateBounds] = NSValue.valueWithMGLCoordinateBounds({
                                sw: CLLocationCoordinate2DMake(options.bounds[1], options.bounds[0]),
                                ne: CLLocationCoordinate2DMake(options.bounds[3], options.bounds[2])
                            });
                        }
                        source = MGLRasterTileSource.alloc().initWithIdentifierTileURLTemplatesOptions(id, options.tiles, sourceOptions);
                        break;
                    }
                    default:
                        reject('Invalid source type: ' + options['type']);
                        return;
                }
                if (!source) {
                    const ex = 'No source to add';
                    if (Trace.isEnabled()) {
                        CLog(CLogTypes.info, 'Error in mapbox.addSource: ' + ex);
                    }
                    reject(ex);
                    return;
                }
                theMap.style.addSource(source);
                resolve();
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.addSource: ' + ex);
                }
                reject(ex);
            }
        });
    }
    /**
     * remove source by id
     */
    removeSource(id, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                if (!theMap) {
                    reject('No map has been loaded');
                    return;
                }
                const source = theMap.style.sourceWithIdentifier(id);
                if (!source) {
                    reject('Source does not exist');
                    return;
                }
                theMap.style.removeSource(source);
                resolve();
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.removeSource: ' + ex);
                }
                reject(ex);
            }
        });
    }
    /**
     * a rough analogue to the mapbox-gl-js addLayer() method
     *
     * It would be nice if this {N} API matched the mapbox-gl-js API which
     * would make it much easier to share mapping applications between the web
     * and {N} apps.
     *
     * This method accepts a Mapbox-GL-JS style specification JSON object with some
     * limitations:
     *
     * - the source: must be a GeoJSON object.
     * - only a subset of paint properties are available.
     *
     * @param {object} style - a style following the Mapbox style specification.
     * @param {any} nativeMapView - native map view (com.mapbox.mapboxsdk.maps.MapView)
     *
     * @link https://docs.mapbox.com/mapbox-gl-js/style-spec/#layers
     */
    async addLayer(style, belowLayerId, nativeMapView) {
        const theMap = nativeMapView || this._mapboxViewInstance;
        let source = null;
        if (typeof style.source != 'string') {
            await this.addSource(style.id + '_source', style.source);
            source = theMap.style.sourceWithIdentifier(style.id + '_source');
        }
        else {
            source = theMap.style.sourceWithIdentifier(style.source);
        }
        const layer = await LayerFactory.createLayer(style, source);
        if (belowLayerId) {
            const belowlayer = theMap.style.layerWithIdentifier(belowLayerId);
            if (belowlayer) {
                theMap.style.insertLayerBelowLayer(layer.getNativeInstance(), belowlayer);
                return;
            }
        }
        theMap.style.addLayer(layer.getNativeInstance());
    }
    /**
     * remove layer by ID
     *
     * Removes a layer given a layer id
     *
     * @param {string} id
     */
    async removeLayer(id, nativeMapViewInstance) {
        const theMap = nativeMapViewInstance || this._mapboxViewInstance;
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, "Mapbox::removeLayer(): attempting to remove layer '" + id + "'");
        }
        const layer = theMap.style.layerWithIdentifier(id);
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'Mapbox:removeLayer(): got layer object: ', layer);
        }
        if (!layer) {
            throw new Error("Layer '" + id + "' not found when attempting to remove it.");
        }
        theMap.style.removeLayer(layer);
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, 'Mapbox:removeLayer(): after removing layer ' + id);
        }
    }
    /**
     * @deprecated
     * Add a point to a line
     *
     * This method appends a point to a line and is useful for drawing a users track.
     *
     * The process for adding a point to a line is different in the iOS sdk than in
     * the Android java sdk.
     *
     * @param {id} id - id of line to add a point to.
     * @param {array} lnglat - [lng,lat] to append to the line.
     *
     * @link https://github.com/mapbox/mapbox-gl-native/issues/13983
     * @link https://docs.mapbox.com/ios/maps/examples/runtime-animate-line/
     */
    async addLinePoint(id, lnglat, sourceId, nativeMapView) {
        const theMap = nativeMapView || this._mapboxViewInstance;
        const sId = !!sourceId ? sourceId : id + '_source';
        const lineSource = theMap.style.sourceWithIdentifier(sId);
        if (!lineSource) {
            throw new Error(`no source found with id: ${sId}`);
        }
        try {
            const lineFeatures = lineSource.featuresMatchingPredicate(ExpressionParser.parseJson(['==', '$type', 'LineString']));
            if (lineFeatures.count === 0) {
                throw new Error('no line string feature found');
            }
            const lineFeature = lineFeatures.objectAtIndex(0);
            const newCoord = CLLocationCoordinate2DMake(lnglat[1], lnglat[0]);
            const newCoordPointer = new interop.Reference(newCoord);
            lineFeature.appendCoordinatesCount(newCoordPointer, 1);
            lineSource.shape = lineFeature;
        }
        catch (error) {
            console.log(error);
            throw error;
        }
    }
    addGeoJsonClustered(options, nativeMapViewInstance) {
        throw new Error('Method not implemented.');
        // return new Promise((resolve, reject) => {
        //     const theMap: MGLMapView = nativeMapViewInstance || this._mapboxViewInstance;
        //     try {
        //         const source = MGLShapeSource.alloc().initWithIdentifierURLOptions(options.name, NSURL.URLWithString(options.data), null);
        //         theMap.style.addSource(source);
        //         const layers = [];
        //         if (options.clusters) {
        //             for (let i = 0; i < options.clusters.length; i++) {
        //                 // TODO also allow Color object
        //                 layers.push([options.clusters[i].points, new Color(options.clusters[i].color).ios]);
        //             }
        //         } else {
        //             layers.push([150, new Color('red').ios]);
        //             layers.push([20, new Color('green').ios]);
        //             layers.push([0, new Color('blue').ios]);
        //         }
        //         const unclustered = MGLCircleStyleLayer.alloc().initWithIdentifierSource(options.name, source);
        //         unclustered.circleColor = NSExpression.expressionWithFormatArgumentArray('%@', new Color('red').ios);
        //         unclustered.circleRadius = NSExpression.expressionWithFormatArgumentArray('16', null);
        //         unclustered.circleBlur = NSExpression.expressionWithFormatArgumentArray('0.2', null);
        //         // unclustered.setFilter(com.mapbox.mapboxsdk.style.expressions.Expression.neq(com.mapbox.mapboxsdk.style.expressions.Expression.get('cluster'), true));
        //         // theMap.style.addLayer(unclustered); // , "building");
        //         for (let i = 0; i < layers.length; i++) {
        //             // Add some nice circles
        //             const circles = MGLCircleStyleLayer.alloc().initWithIdentifierSource(options.name, source);
        //             const circles = new com.mapbox.mapboxsdk.style.layers.CircleLayer('cluster-' + i, options.name);
        //         //     circles.setProperties([
        //         //         // com.mapbox.mapboxsdk.style.layers.PropertyFactory.iconImage("icon")
        //         //         com.mapbox.mapboxsdk.style.layers.PropertyFactory.circleColor(layers[i][1]),
        //         //         com.mapbox.mapboxsdk.style.layers.PropertyFactory.circleRadius(new java.lang.Float(22.0)),
        //         //         com.mapbox.mapboxsdk.style.layers.PropertyFactory.circleBlur(new java.lang.Float(0.2)),
        //         //     ]);
        //         //     const pointCount = com.mapbox.mapboxsdk.style.expressions.Expression.toNumber(com.mapbox.mapboxsdk.style.expressions.Expression.get('point_count'));
        //         //     if (i === 0) {
        //         //         circles.setFilter(
        //         //             com.mapbox.mapboxsdk.style.expressions.Expression.gte(pointCount, com.mapbox.mapboxsdk.style.expressions.Expression.literal(java.lang.Integer.valueOf(layers[i][0])))
        //         //         );
        //         //     } else {
        //         //         circles.setFilter(
        //         //             com.mapbox.mapboxsdk.style.expressions.Expression.all([
        //         //                 com.mapbox.mapboxsdk.style.expressions.Expression.gte(pointCount, com.mapbox.mapboxsdk.style.expressions.Expression.literal(java.lang.Integer.valueOf(layers[i][0]))),
        //         //                 com.mapbox.mapboxsdk.style.expressions.Expression.lt(
        //         //                     pointCount,
        //         //                     com.mapbox.mapboxsdk.style.expressions.Expression.literal(java.lang.Integer.valueOf(layers[i - 1][0]))
        //         //                 ),
        //         //             ])
        //         //         );
        //         //     }
        //         //     this._mapboxMapInstance.getStyle().addLayer(circles); // , "building");
        //         }
        //         // // Add the count labels (note that this doesn't show.. #sad)
        //         // const count = new com.mapbox.mapboxsdk.style.layers.SymbolLayer('count', options.name);
        //         // count.setProperties([
        //         //     com.mapbox.mapboxsdk.style.layers.PropertyFactory.textField(com.mapbox.mapboxsdk.style.expressions.Expression.get('point_count')),
        //         //     com.mapbox.mapboxsdk.style.layers.PropertyFactory.textSize(new java.lang.Float(12.0)),
        //         //     com.mapbox.mapboxsdk.style.layers.PropertyFactory.textColor(new Color('white').android),
        //         // ]);
        //         // this._mapboxMapInstance.getStyle().addLayer(count);
        //         resolve();
        //     } catch (ex) {
        //         if (Trace.isEnabled()) {
        //             CLog(CLogTypes.info, 'Error in mapbox.addGeoJsonClustered: ' + ex);
        //         }
        //         reject(ex);
        //     }
        // });
    }
    trackUser(options, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                if (!theMap) {
                    reject('No map has been loaded');
                    return;
                }
                if (!theMap.showsUserLocation) {
                    reject('The map is not currently showing the user location');
                    return;
                }
                theMap.setUserTrackingModeAnimated(this._stringToCameraMode(options.cameraMode), options.animated !== false);
                resolve();
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.trackUser: ' + ex);
                }
                reject(ex);
            }
        });
    }
    getLayer(name, nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                if (!theMap) {
                    reject('No map has been loaded');
                    return;
                }
                const layer = theMap.style.layerWithIdentifier(name);
                resolve(layer ? new Layer(layer) : null);
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.getLayer: ' + ex);
                }
                reject(ex);
            }
        });
    }
    getLayers(nativeMap) {
        return new Promise((resolve, reject) => {
            try {
                const theMap = nativeMap || this._mapboxViewInstance;
                if (!theMap) {
                    reject('No map has been loaded');
                    return;
                }
                const layers = theMap.style.layers;
                const result = [];
                for (let i = 0; i < layers.count; i++) {
                    result.push(new Layer(layers[i]));
                }
                resolve(result);
            }
            catch (ex) {
                if (Trace.isEnabled()) {
                    CLog(CLogTypes.info, 'Error in mapbox.getLayers: ' + ex);
                }
                reject(ex);
            }
        });
    }
    project(data) {
        const theMap = this._mapboxViewInstance;
        const { x, y } = theMap.convertCoordinateToPointToView({ latitude: data.lat, longitude: data.lng }, theMap);
        return { x, y };
    }
    projectBack(screenCoordinate) {
        const theMap = this._mapboxViewInstance;
        const cgPoint = {
            x: screenCoordinate.x,
            y: screenCoordinate.y
        };
        const coordinate = theMap.convertPointToCoordinateFromView(cgPoint, theMap);
        return {
            lat: coordinate.latitude,
            lng: coordinate.longitude
        };
    }
    getUserLocationCameraMode(nativeMap) {
        const theMap = nativeMap || this._mapboxViewInstance;
        if (!theMap) {
            return 'NONE';
        }
        return this._convertCameraMode(theMap.userTrackingMode);
    }
}
const _addObserver = (eventName, callback) => NSNotificationCenter.defaultCenter.addObserverForNameObjectQueueUsingBlock(eventName, null, NSOperationQueue.mainQueue, callback);
function _downloadImage(marker) {
    return new Promise((resolve, reject) => {
        if (Trace.isEnabled()) {
            CLog(CLogTypes.info, '>> _downloadImage');
        }
        // to cache..
        if (_markerIconDownloadCache[marker.icon]) {
            marker.iconDownloaded = _markerIconDownloadCache[marker.icon];
            if (Trace.isEnabled()) {
                CLog(CLogTypes.info, '>> marker.iconDownloaded: ' + marker.iconDownloaded);
            }
            resolve(marker);
            return;
        }
        // ..or not to cache
        Http.getImage(marker.icon).then((output) => {
            marker.iconDownloaded = output.ios;
            _markerIconDownloadCache[marker.icon] = marker.iconDownloaded;
            resolve(marker);
        }, (ignoredError) => {
            console.log(`Download failed for ${marker.icon} with error: ${ignoredError}`);
            resolve(marker);
        });
    });
}
const _downloadMarkerImages = (markers) => {
    const iterations = [];
    const result = [];
    markers.forEach((marker) => {
        if (marker.icon && marker.icon.startsWith('http')) {
            const p = _downloadImage(marker).then((mark) => result.push(mark));
            iterations.push(p);
        }
        else {
            result.push(marker);
        }
    });
    return Promise.all(iterations).then(() => result);
};
//# sourceMappingURL=index.ios.js.map