import { ExpressionParser } from '../expression/expression-parser';
import { PropertyParser } from './parser/property-parser';
export class LayerFactory {
    static async createLayer(style, source) {
        let nativeLayer;
        switch (style.type) {
            case 'line':
                nativeLayer = MGLLineStyleLayer.alloc().initWithIdentifierSource(style.id, source);
                break;
            case 'circle':
                nativeLayer = MGLCircleStyleLayer.alloc().initWithIdentifierSource(style.id, source);
                break;
            case 'fill':
                nativeLayer = MGLFillStyleLayer.alloc().initWithIdentifierSource(style.id, source);
                break;
            case 'symbol':
                nativeLayer = MGLSymbolStyleLayer.alloc().initWithIdentifierSource(style.id, source);
                break;
            case 'raster':
                nativeLayer = MGLRasterStyleLayer.alloc().initWithIdentifierSource(style.id, source);
                break;
            default:
                throw new Error(`Unknown layer type: ${style.type}`);
        }
        if (style.minzoom !== undefined) {
            nativeLayer.minimumZoomLevel = style.minzoom;
        }
        if (style.maxzoom !== undefined) {
            nativeLayer.maximumZoomLevel = style.maxzoom;
        }
        if (style['source-layer']) {
            nativeLayer.sourceLayerIdentifier = style['source-layer'];
        }
        const layerProperties = this.parseProperties(style.type, Object.assign(style.paint || {}, style.layout || {})); // TODO: handle defaults
        for (const propKey in layerProperties) {
            if (Object.prototype.hasOwnProperty.call(layerProperties, propKey)) {
                nativeLayer[propKey] = layerProperties[propKey];
            }
        }
        const layer = new Layer(nativeLayer);
        return layer;
    }
    static parseProperties(layerType, propertiesObject) {
        return PropertyParser.parsePropertiesForLayer(propertiesObject);
    }
}
export class Layer {
    constructor(instance) {
        this.instance = instance;
        this.id = instance.identifier;
    }
    type() {
        if (this.instance instanceof MGLFillStyleLayer) {
            return "fill";
        }
        if (this.instance instanceof MGLLineStyleLayer) {
            return "line";
        }
        if (this.instance instanceof MGLSymbolStyleLayer) {
            return "symbol";
        }
        if (this.instance instanceof MGLCircleStyleLayer) {
            return "circle";
        }
        if (this.instance instanceof MGLHeatmapStyleLayer) {
            return "heatmap";
        }
        if (this.instance instanceof MGLFillExtrusionStyleLayer) {
            return "fill-extrusion";
        }
        if (this.instance instanceof MGLRasterStyleLayer) {
            return "raster";
        }
        if (this.instance instanceof MGLHillshadeStyleLayer) {
            return "hillshade";
        }
        if (this.instance instanceof MGLBackgroundStyleLayer) {
            return "background";
        }
        // there is no sky layer in the Mapbox iOS SDK
        return null;
    }
    visibility() {
        return this.instance.visible;
    }
    show() {
        this.instance.visible = true;
    }
    hide() {
        this.instance.visible = false;
    }
    getNativeInstance() {
        return this.instance;
    }
    setFilter(filter) {
        if (this.instance instanceof MGLVectorStyleLayer) {
            // MGLVectorStyleLayer is the base type of many layer types. Predicates only supported on vector style layers.
            // See https://docs.mapbox.com/ios/maps/api/6.3.0/Classes/MGLVectorStyleLayer.html
            this.instance.predicate = ExpressionParser.parseJson(filter);
        }
        else {
            throw new Error('Set filter only support for vector layer.');
        }
    }
    getFilter() {
        return ExpressionParser.toJson(this.instance.predicate);
    }
    setProperty(name, value) {
        const properties = PropertyParser.parsePropertiesForLayer({ [name]: value });
        for (const propKey in properties) {
            if (Object.prototype.hasOwnProperty.call(properties, propKey)) {
                this.instance[propKey] = properties[propKey];
            }
        }
    }
    getProperty(name) {
        return PropertyParser.propertyValueFromLayer(this.instance, name);
    }
}
//# sourceMappingURL=layer-factory.ios.js.map